﻿package wbEVPkg "Package containing basic EV models"
  package SupportModels
    model DragForce "Vehicle rolling and aerodinamical drag force"
      import Modelica.Constants.g_n;
      extends Modelica.Mechanics.Translational.Interfaces.PartialElementaryOneFlangeAndSupport2;
      extends Modelica.Mechanics.Translational.Interfaces.PartialFriction;
      Modelica.SIunits.Force f "Total drag force";
      Modelica.SIunits.Velocity v "vehicle velocity";
      Modelica.SIunits.Acceleration a "Absolute acceleration of flange";
      Real Sign;
      parameter Modelica.SIunits.Mass m "vehicle mass";
      parameter Modelica.SIunits.Density rho = 1.226 "air density";
      parameter Modelica.SIunits.Area S "vehicle cross area";
      parameter Real fc(start = 0.01) "rolling friction coefficient";
      parameter Real Cx "aerodinamic drag coefficient";
    protected
      parameter Real A = fc * m * g_n;
      parameter Real B = 1 / 2 * rho * S * Cx;
    equation
//  s = flange.s;
      v = der(s);
      a = der(v);
// Le seguenti definizioni seguono l'ordine e le richieste del modello "PartialFriction" di
// Modelica.Mechanics.Translational.Interfaces"
      v_relfric = v;
      a_relfric = a;
      f0 = A "force at 0 speed 0 but with slip";
      f0_max = A "max force at 0 speed without slip";
      free = false "in principle should become true whenthe wheel loose contact with road";
// Now the computation of f, and its attribution to the flange:
      flange.f - f = 0;
// friction force
      if v > 0 then
        Sign = 1;
      else
        Sign = -1;
      end if;
      f - B * v ^ 2 * Sign = if locked then sa * unitForce else f0 * (if startForward then Modelica.Math.tempInterpol1(v, [0, 1], 2) else if startBackward then -Modelica.Math.tempInterpol1(-v, [0, 1], 2) else if pre(mode) == Forward then Modelica.Math.tempInterpol1(v, [0, 1], 2) else -Modelica.Math.tempInterpol1(-v, [0, 1], 2));
      annotation(
        Documentation(info = "<html>
<p>This component models the total (rolling and aerodynamic) vehicle drag resistance: </p>
<p>F=fc*m*g+(1/2)*rho*Cx*S*v^2 </p>
<p>It models reliably the stuck phase. Based on Modelica-Intrerfaces.PartialFriction model </p>
</html>"),
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics = {Polygon(points = {{-98, 10}, {22, 10}, {22, 41}, {92, 0}, {22, -41}, {22, -10}, {-98, -10}, {-98, 10}}, lineColor = {0, 127, 0}, fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid), Line(points = {{-42, -50}, {87, -50}}, color = {0, 0, 0}), Polygon(points = {{-72, -50}, {-41, -40}, {-41, -60}, {-72, -50}}, lineColor = {0, 0, 0}, fillColor = {128, 128, 128}, fillPattern = FillPattern.Solid), Line(points = {{-90, -90}, {-70, -88}, {-50, -82}, {-30, -72}, {-10, -58}, {10, -40}, {30, -18}, {50, 8}, {70, 38}, {90, 72}, {110, 110}}, color = {0, 0, 255}, thickness = 0.5), Text(extent = {{-82, 90}, {80, 50}}, lineColor = {0, 0, 255}, textString = "%name")}),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics));
    end DragForce;

    block PropDriver "Simple Proportional controller driver"
      parameter String CycleFileName = "cycleName.txt" "Drive Cycle Name ex: \"sort1.txt\"";
      parameter Real k "Controller gain";
      parameter Real yMax = 1.e6 "Max output value (absolute)";
      parameter Modelica.Blocks.Types.Extrapolation extrapolation = Modelica.Blocks.Types.Extrapolation.LastTwoPoints "Extrapolation of data outside the definition range";
      Modelica.Blocks.Interfaces.RealInput V annotation(
        Placement(visible = true, transformation(origin = {0, -66}, extent = {{-14, -14}, {14, 14}}, rotation = 90), iconTransformation(origin = {0, -112}, extent = {{-12, -12}, {12, 12}}, rotation = 90)));
      Modelica.Blocks.Math.UnitConversions.From_kmh from_kmh annotation(
        Placement(visible = true, transformation(extent = {{-42, -10}, {-22, 10}}, rotation = 0)));
      Modelica.Blocks.Sources.CombiTimeTable driveCyc(columns = {2}, extrapolation = extrapolation, fileName = CycleFileName, tableName = "Cycle", tableOnFile = true) annotation(
        Placement(visible = true, transformation(extent = {{-80, -10}, {-60, 10}}, rotation = 0)));
      Modelica.Blocks.Math.Feedback feedback annotation(
        Placement(visible = true, transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Blocks.Math.Gain gain(k = k) annotation(
        Placement(visible = true, transformation(extent = {{14, -10}, {34, 10}}, rotation = 0)));
      Modelica.Blocks.Nonlinear.Limiter limAcc(uMax = yMax, uMin = 0) annotation(
        Placement(visible = true, transformation(origin = {2, 40}, extent = {{52, -10}, {72, 10}}, rotation = 0)));
      Modelica.Blocks.Nonlinear.Limiter limBrak(uMax = 0, uMin = -yMax) annotation(
        Placement(visible = true, transformation(origin = {0, -40}, extent = {{52, -10}, {72, 10}}, rotation = 0)));
      Modelica.Blocks.Interfaces.RealOutput tauRef(unit = "N.m") annotation(
        Placement(visible = true, transformation(extent = {{100, -10}, {120, 10}}, rotation = 0), iconTransformation(extent = {{100, -10}, {120, 10}}, rotation = 0)));
      Modelica.Blocks.Interfaces.RealOutput accelTau annotation(
        Placement(visible = true, transformation(origin = {110, 40}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(extent = {{-10, 10}, {10, 30}}, rotation = 0)));
      Modelica.Blocks.Interfaces.RealOutput brakeTau annotation(
        Placement(visible = true, transformation(origin = {110, -40}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(extent = {{-10, -30}, {10, -10}}, rotation = 0)));
    equation
      connect(V, feedback.u2) annotation(
        Line(points = {{0, -66}, {0, -66}, {0, -8}, {0, -8}}, color = {0, 0, 127}));
      connect(from_kmh.u, driveCyc.y[1]) annotation(
        Line(points = {{-44, 0}, {-59, 0}}, color = {0, 0, 127}));
      connect(from_kmh.y, feedback.u1) annotation(
        Line(points = {{-21, 0}, {-8, 0}}, color = {0, 0, 127}));
      connect(feedback.y, gain.u) annotation(
        Line(points = {{9, 0}, {12, 0}}, color = {0, 0, 127}));
      connect(limBrak.y, brakeTau) annotation(
        Line(points = {{73, -40}, {104, -40}, {104, -40}, {110, -40}}, color = {0, 0, 127}));
      connect(limAcc.y, accelTau) annotation(
        Line(points = {{75, 40}, {102, 40}, {102, 40}, {110, 40}}, color = {0, 0, 127}));
      connect(limBrak.u, gain.y) annotation(
        Line(points = {{50, -40}, {40, -40}, {40, 0}, {35, 0}, {35, 0}}, color = {0, 0, 127}));
      connect(limAcc.u, gain.y) annotation(
        Line(points = {{52, 40}, {40, 40}, {40, 0}, {35, 0}, {35, 0}}, color = {0, 0, 127}));
      connect(gain.y, tauRef) annotation(
        Line(points = {{35, 0}, {110, 0}, {110, 0}}, color = {0, 0, 127}));
      annotation(
        Documentation(info = "<html><head></head><body><p>Simple driver model.</p><p>It reads a reference cycle from a file then controls speed with a simple proportional feedback law.</p>
            </body></html>"),
        Icon(coordinateSystem(preserveAspectRatio = false, initialScale = 0.1), graphics = {Rectangle(fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, 100}, {100, -100}}), Ellipse(fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, extent = {{-23, 22}, {-12, -4}}, endAngle = 360), Text(origin = {2, -0.1894}, lineColor = {0, 0, 255}, extent = {{-104, 142.189}, {98, 104}}, textString = "%name"), Polygon(fillColor = {215, 215, 215}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-22, -60}, {-42, -88}, {-16, -88}, {16, -88}, {-22, -60}}), Polygon(fillColor = {135, 135, 135}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-32, 40}, {-62, -52}, {-30, -52}, {-30, -52}, {-32, 40}}, smooth = Smooth.Bezier), Polygon(fillColor = {135, 135, 135}, pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-68, -36}, {-14, -90}, {10, -50}, {0, -50}, {-68, -36}}, smooth = Smooth.Bezier), Polygon(fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid, points = {{-22, 10}, {-30, 6}, {-40, -48}, {2, -46}, {2, -34}, {0, 2}, {-22, 10}}, smooth = Smooth.Bezier), Ellipse(fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, extent = {{-30, 44}, {-3, 10}}, endAngle = 360), Polygon(pattern = LinePattern.None, fillPattern = FillPattern.Solid, points = {{-38, 34}, {-16, 50}, {-2, 36}, {4, 36}, {6, 36}, {-38, 34}}, smooth = Smooth.Bezier), Polygon(fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, points = {{30, -44}, {-32, -28}, {-36, -44}, {-24, -58}, {30, -44}}, smooth = Smooth.Bezier), Polygon(fillPattern = FillPattern.Solid, points = {{42, -70}, {36, -84}, {48, -78}, {52, -72}, {50, -68}, {42, -70}}, smooth = Smooth.Bezier), Line(points = {{48, -14}, {26, 0}, {26, 0}}, thickness = 0.5), Line(points = {{20, -10}, {34, 10}, {34, 10}}, thickness = 0.5), Polygon(fillColor = {255, 213, 170}, fillPattern = FillPattern.Solid, points = {{28, 4}, {32, 8}, {28, 2}, {34, 6}, {30, 2}, {34, 4}, {30, 0}, {26, 2}, {34, 0}, {26, 0}, {26, 2}, {28, 4}, {28, 4}, {26, 2}, {26, 2}, {26, 2}, {28, 8}, {28, 6}, {28, 4}}, smooth = Smooth.Bezier), Polygon(fillColor = {175, 175, 175}, fillPattern = FillPattern.Solid, points = {{-18, 0}, {28, 6}, {26, -2}, {-16, -16}, {-20, -16}, {-24, -6}, {-18, 0}}, smooth = Smooth.Bezier), Polygon(fillColor = {215, 215, 215}, fillPattern = FillPattern.Solid, points = {{72, -6}, {48, -6}, {36, -26}, {58, -86}, {72, -86}, {72, -6}}), Polygon(fillColor = {95, 95, 95}, fillPattern = FillPattern.Solid, points = {{49, -94}, {17, -40}, {7, -44}, {-1, -50}, {49, -94}}, smooth = Smooth.Bezier), Line(points = {{-7, 31}, {-3, 29}}), Line(points = {{-9, 18}, {-5, 18}}), Line(points = {{-7, 31}, {-3, 31}}), Text(lineColor = {238, 46, 47}, extent = {{-100, 90}, {100, 58}}, textString = "%CycleFileName")}),
        Diagram(coordinateSystem(extent = {{-100, -60}, {100, 60}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})));
    end PropDriver;

    model Batt1 "Battery model based on one R-C block in its electric circuit"
      parameter Modelica.SIunits.ElectricCharge QCellNom(min = 0) = 10 * 3600.0 "Nominal electric charge" annotation(
        Dialog(tab = "Cell data"));
      parameter Modelica.SIunits.Voltage ECellMin(min = 0) = 3.3 "Minimum open source voltage" annotation(
        Dialog(tab = "Cell data"));
      parameter Modelica.SIunits.Voltage ECellMax(min = 0.0001) = 4.15 "Maximum open source voltage" annotation(
        Dialog(tab = "Cell data"));
      parameter Real SOCMin(min = 0, max = 1) = 0 "Minimum state of charge" annotation(
        Dialog(group = "SOC parameters"));
      parameter Real SOCMax(min = 0, max = 1) = 1 "Maximum state of charge" annotation(
        Dialog(group = "SOC parameters"));
      parameter Real SOCInit(min = 0, max = 1) = 0.5 "Initial state of charge" annotation(
        Dialog(group = "SOC parameters"));
      parameter Modelica.SIunits.Current ICellMax(min = 0) = 10 * QCellNom / 3600.0 "Maximum admissible current" annotation(
        Dialog(tab = "Cell data"));
      parameter Modelica.SIunits.Resistance R0Cell(min = 0) = 0.05 * ECellMax / ICellMax "Serial resistance \"R0\"" annotation(
        Dialog(tab = "Cell data", group = "Electric circuit parameters"));
      parameter Modelica.SIunits.Resistance R1Cell(min = 0) = R0Cell "Serial resistance \"R1\"" annotation(
        Dialog(tab = "Cell data", group = "Electric circuit parameters"));
      parameter Modelica.SIunits.Capacitance C1Cell(min = 0) = 60 / R1Cell "Capacitance in parallel with R1" annotation(
        Dialog(tab = "Cell data", group = "Electric circuit parameters"));
      parameter Real efficiency(min = 0, max = 0.9999) = 0.85 "Overall charging/discharging energy efficiency" annotation(
        Dialog(group = "Parameters related to losses"));
      parameter Modelica.SIunits.Current iCellEfficiency(min = 0) = 0.5 * ICellMax "Charging/discharging current the efficiency refers to" annotation(
        Dialog(group = "Parameters related to losses"));
      parameter Integer ns = 1 "Number of serial connected cells per string" annotation(
        Dialog(tab = "Battery pack data", group = "Size of the package"));
      parameter Integer np = 1 "Number of parallel connected strings" annotation(
        Dialog(tab = "Battery pack data", group = "Size of the package"));
      Modelica.SIunits.Voltage Ubat(start = EBatteryMin + SOCInit * (EBatteryMax - EBatteryMin), fixed = true);
      Modelica.SIunits.Power powerLoss;
      Modelica.Electrical.Analog.Basic.Capacitor cBattery(final C = CBattery) annotation(
        Placement(transformation(origin = {-60, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
      Modelica.Electrical.Analog.Basic.Resistor R0(final R = R0Battery) annotation(
        Placement(transformation(origin = {20, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
      Modelica.Electrical.Analog.Sources.SignalCurrent Ip annotation(
        Placement(transformation(origin = {-6, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
      Modelica.Electrical.Analog.Interfaces.Pin p annotation(
        Placement(transformation(extent = {{90, 50}, {110, 70}}), iconTransformation(extent = {{90, 50}, {110, 70}})));
      Modelica.Electrical.Analog.Interfaces.NegativePin n annotation(
        Placement(transformation(extent = {{90, -70}, {110, -50}}), iconTransformation(extent = {{91, -70}, {111, -50}})));
      Modelica.Electrical.Analog.Basic.Resistor R1(final R = R1Battery) annotation(
        Placement(transformation(origin = {-37, 74}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
      Modelica.Electrical.Analog.Basic.Capacitor C1(C = C1Battery) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-37, 50})));
      Modelica.Blocks.Interfaces.RealOutput SOC annotation(
        Placement(visible = true, transformation(origin = {-90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 180), iconTransformation(origin = {-110, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
      Modelica.Electrical.Analog.Basic.Capacitor cDummy(C = C1Battery / 10000) annotation(
        Placement(visible = true, transformation(origin = {88, 0}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
    protected
      parameter Real efficiencyMax = (EBatteryMin + EBatteryMax - 2 * Rtot * iCellEfficiency) / (EBatteryMin + EBatteryMax + 2 * Rtot * iCellEfficiency);
      parameter Modelica.SIunits.Capacitance C = QCellNom / (ECellMax - ECellMin) "Cell capacitance";
      // determine fraction of drain current with respect to the total package current
      parameter Real k = ((1 - efficiency) * (EBatteryMax + EBatteryMin) - 2 * (1 + efficiency) * Rtot * iCellEfficiency) / ((1 + efficiency) * (EBatteryMax + EBatteryMin) - 2 * (1 - efficiency) * Rtot * iCellEfficiency);
      parameter Modelica.SIunits.Current IBatteryMax = ICellMax * np "Maximum battery current";
      parameter Modelica.SIunits.Voltage EBatteryMin = ECellMin * ns "Minimum battery voltage";
      parameter Modelica.SIunits.Voltage EBatteryMax = ECellMax * ns "Maximum battery voltage";
      parameter Modelica.SIunits.ElectricCharge QBatteryNominal = QCellNom * np "Battery admissible electric charge";
      parameter Modelica.SIunits.Capacitance CBattery = C * np / ns "Battery capacitance";
      parameter Modelica.SIunits.Resistance R0Battery = R0Cell * ns / np "Serial inner resistance R0 of cell package";
      parameter Modelica.SIunits.Resistance R1Battery = R1Cell * ns / np "Serial inner resistance R1 of cell package";
      parameter Modelica.SIunits.Resistance Rtot = R0Battery + R1Battery;
      parameter Modelica.SIunits.Capacitance C1Battery = C1Cell * np / ns "Battery series inner capacitance C1";
    protected
      Modelica.SIunits.Voltage ECell "Cell e.m.f.";
      Modelica.SIunits.Current iCellStray "Cell stray current";
      Modelica.SIunits.Voltage EBattery(start = EBatteryMin + SOCInit * (EBatteryMax - EBatteryMin), fixed = true) "Battery e.m.f.";
      Modelica.SIunits.Current iBatteryStray "Cell parasitic current";
      Modelica.Electrical.Analog.Sensors.CurrentSensor currentSensor annotation(
        Placement(transformation(extent = {{60, 50}, {80, 70}}, rotation = 0)));
      Modelica.Blocks.Math.Gain gain(k = k) annotation(
        Placement(transformation(origin = {52, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
      Modelica.Blocks.Math.Abs abs1 annotation(
        Placement(transformation(extent = {{34, -10}, {14, 10}}, rotation = 0)));
    equation
      connect(cDummy.n, n) annotation(
        Line(points = {{88, -10}, {88, -10}, {88, -60}, {100, -60}, {100, -60}}, color = {0, 0, 255}));
      connect(cDummy.p, currentSensor.n) annotation(
        Line(points = {{88, 10}, {88, 10}, {88, 60}, {80, 60}, {80, 60}}, color = {0, 0, 255}));
      assert(SOCMin >= 0, "SOCMin must be greater than, or equal to 0");
      assert(SOCMax <= 1, "SOCMax must be smaller than, or equal to 1");
      assert(efficiency <= efficiencyMax, "Overall charging/discharging energy efficiency is too big with respect to the actual serial resistance (EfficiencyMax =" + String(efficiencyMax) + ")");
      assert(SOCMin < SOCMax, "SOCMax(=" + String(SOCMax) + ") must be greater than SOCMin(=" + String(SOCMin) + ")");
      assert(SOCInit >= SOCMin, "SOCInit(=" + String(SOCInit) + ") must be greater than, or equal to SOCMin(=" + String(SOCMin) + ")");
      assert(SOCInit <= SOCMax, "SOCInit(=" + String(SOCInit) + ") must be smaller than, or equal to SOCMax(=" + String(SOCMax) + ")");
      iBatteryStray = Ip.i;
      iCellStray = iBatteryStray / np;
      EBattery = cBattery.v;
//Solo per dare maggiore chiarezza all'utente con un nome significativo
      Ubat = p.v - n.v;
      powerLoss = R0.LossPower + R1.LossPower + Ip.v * Ip.i;
      ECell = EBattery / ns;
      assert(abs(p.i / np) < ICellMax, "Battery cell current i=" + String(abs(p.i / np)) + "\n exceeds max admissable ICellMax (=" + String(ICellMax) + "A)");
      SOC = (EBattery - EBatteryMin) / (EBatteryMax - EBatteryMin);
//*(SOCMax-SOCMin)+SOCMin);
      assert(SOC <= SOCMax, "Battery is fully charged: State of charge reached maximum limit (=" + String(SOCMax) + ")");
      assert(SOCMin <= SOC, "Battery is fully discharged: State of charge reached minimum limit (=" + String(SOCMin) + ")");
      connect(R0.p, currentSensor.p) annotation(
        Line(points = {{30, 60}, {60, 60}}, color = {0, 0, 255}));
      connect(Ip.p, R0.n) annotation(
        Line(points = {{-6, 10}, {-6, 60}, {10, 60}}, color = {0, 0, 255}));
      connect(currentSensor.i, gain.u) annotation(
        Line(points = {{70, 50}, {70, -1.46958e-015}, {64, -1.46958e-015}}, color = {0, 0, 127}));
      connect(abs1.u, gain.y) annotation(
        Line(points = {{36, 0}, {39.5, 0}, {39.5, 1.34711e-015}, {41, 1.34711e-015}}, color = {0, 0, 127}));
      connect(abs1.y, Ip.i) annotation(
        Line(points = {{13, 0}, {7, 0}, {7, -1.28588e-015}, {1, -1.28588e-015}}, color = {0, 0, 127}));
      connect(currentSensor.n, p) annotation(
        Line(points = {{80, 60}, {80, 60}, {100, 60}}, color = {0, 0, 255}));
      connect(Ip.n, n) annotation(
        Line(points = {{-6, -10}, {-6, -60}, {100, -60}}, color = {0, 0, 255}));
      connect(n, cBattery.n) annotation(
        Line(points = {{100, -60}, {-60, -60}, {-60, -10}}, color = {0, 0, 255}));
      connect(R1.n, cBattery.p) annotation(
        Line(points = {{-47, 74}, {-60, 74}, {-60, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
      connect(C1.n, cBattery.p) annotation(
        Line(points = {{-47, 50}, {-60, 50}, {-60, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
      connect(R1.p, C1.p) annotation(
        Line(points = {{-27, 74}, {-18, 74}, {-18, 50}, {-27, 50}}, color = {0, 0, 255}, smooth = Smooth.None));
      connect(R1.p, R0.n) annotation(
        Line(points = {{-27, 74}, {-18, 74}, {-18, 60}, {10, 60}}, color = {0, 0, 255}, smooth = Smooth.None));
      annotation(
        Documentation(info = "<html>
<p>Battery model with non-unity coulombic efficiency. </p>
<p>The main cell branch contains an e.m.f. that is linearly increasing with SOC, simulated through an equivalent capacitor, the resistance R0 and a parallel R-C couple. </p>
<p>The full battery is composed by np rows in parallel, each of them containing ns cells in series.</p>
</html>", revisions = "<html><table border=\"1\" rules=\"groups\">
    <thead>
    <tr><td>Version</td>  <td>Date</td>  <td>Comment</td></tr>
    </thead>
    <tbody>
    <tr><td>1.0.0</td>  <td>2006-01-12</td>  <td> </td></tr>
    <tr><td>1.0.3</td>  <td>2006-08-31</td>  <td> Improved assert statements </td></tr>
    <tr><td>1.0.6</td>  <td>2007-05-14</td>  <td> The documentation changed slightly </td></tr>
    </tbody>
    </table>
    </html>"),
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}, grid = {2, 2}), graphics),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(lineColor = {95, 95, 95}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, 100}, {78, -100}}), Line(origin = {2, -2}, points = {{-92, 7}, {-56, 7}}, color = {0, 0, 255}), Rectangle(lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, extent = {{-82, -3}, {-65, -10}}), Line(points = {{-73, 63}, {98, 64}}, color = {0, 0, 255}), Rectangle(lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{38, 69}, {68, 57}}), Rectangle(lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-37.5, 68}, {-6.5, 56}}), Line(points = {{-19.5, 49}, {-19.5, 32}}, color = {0, 0, 255}), Line(points = {{-54.5, 63}, {-54.5, 41}, {-25.5, 41}}, color = {0, 0, 255}), Line(points = {{9.5, 62}, {9.5, 40}, {-19.5, 40}}, color = {0, 0, 255}), Line(points = {{-73, 63}, {-73, 5}}, color = {0, 0, 255}), Line(points = {{-73, -6}, {-73, -60}, {96, -60}}, color = {0, 0, 255}), Line(points = {{26, 63}, {26, -61}}, color = {0, 0, 255}), Line(points = {{-25.5, 49}, {-25.5, 32}}, color = {0, 0, 255}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{26, 22}, {14, 4}, {26, -14}, {38, 4}, {26, 22}}), Line(points = {{20, 4}, {32, 4}}, color = {0, 0, 255}), Polygon(lineColor = {0, 0, 255}, points = {{22, -20}, {30, -20}, {26, -32}, {22, -20}}), Text(lineColor = {0, 0, 255}, extent = {{-100, 150}, {100, 110}}, textString = "%name")}));
    end Batt1;

    model Batt1Conn "Battery model based on Batt0 but with electric dynamics order = 1"
      Real powDeliv "battery power (positive when delivered)";
      Real SOC "State Of Charge";
      parameter Modelica.SIunits.ElectricCharge QCellNom(min = 0) = 10 * 3.6e3 "Nominal admissible electric charge per cell" annotation(
        Dialog(tab = "Cell data"));
      parameter Modelica.SIunits.Voltage ECellMin(min = 0) = 3.3 "Minimum open source voltage per cell" annotation(
        Dialog(tab = "Cell data"));
      parameter Modelica.SIunits.Voltage ECellMax(min = 0.0001) = 4.15 "Maximum open source voltage per cell" annotation(
        Dialog(tab = "Cell data"));
      parameter Real SOCMin(min = 0, max = 1) = 0 "Minimum state of charge" annotation(
        Dialog(group = "SOC parameters"));
      parameter Real SOCMax(min = 0, max = 1) = 1 "Maximum state of charge" annotation(
        Dialog(group = "SOC parameters"));
      parameter Real SOCInit(min = 0, max = 1) = 0.5 "Initial state of charge" annotation(
        Dialog(group = "SOC parameters"));
      parameter Modelica.SIunits.Current ICellMax(min = 0) = 10 * QCellNom / 3.6e3 "Maximum admissible current" annotation(
        Dialog(tab = "Cell data"));
      parameter Modelica.SIunits.Resistance R0Cell(min = 0) = 0.05 * ECellMax / ICellMax "Series resistance \"R0\"" annotation(
        Dialog(tab = "Cell data", group = "Electric circuit parameters"));
      parameter Modelica.SIunits.Resistance R1Cell(min = 0) = R0Cell "Series resistance \"R1\"" annotation(
        Dialog(tab = "Cell data", group = "Electric circuit parameters"));
      parameter Modelica.SIunits.Capacitance C1Cell(min = 0) = 60 / R1Cell "Capacitance in parallel with R1" annotation(
        Dialog(tab = "Cell data", group = "Electric circuit parameters"));
      parameter Real efficiency(min = 0, max = 0.9999) = 0.85 "Overall charging/discharging energy efficiency" annotation(
        Dialog(group = "Parameters related to losses"));
      parameter Modelica.SIunits.Current iCellEfficiency(min = 0) = 0.5 * ICellMax "Cell charging/discharging current the efficiency refers to" annotation(
        Dialog(group = "Parameters related to losses"));
      parameter Integer ns = 1 "Number of serial connected cells" annotation(
        Dialog(tab = "Battery Pack data", group = "Size of the package"));
      parameter Integer np = 1 "Number of parallel connected cells" annotation(
        Dialog(tab = "Battery Pack data", group = "Size of the package"));
      // determine fraction of drain current with respect to the total package current
      Modelica.Electrical.Analog.Basic.Capacitor cBattery(final C = CBattery) annotation(
        Placement(transformation(origin = {-60, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
      Modelica.Electrical.Analog.Basic.Resistor R0(final R = R0Battery) annotation(
        Placement(transformation(origin = {20, 60}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
      Modelica.Electrical.Analog.Sources.SignalCurrent Ip annotation(
        Placement(transformation(origin = {-6, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
      Modelica.Electrical.Analog.Interfaces.Pin p annotation(
        Placement(transformation(extent = {{90, 50}, {110, 70}}), iconTransformation(extent = {{90, 52}, {110, 72}})));
      Modelica.Electrical.Analog.Interfaces.NegativePin n annotation(
        Placement(transformation(extent = {{90, -70}, {110, -50}}), iconTransformation(extent = {{91, -70}, {111, -50}})));
      Modelica.Electrical.Analog.Basic.Capacitor C1(C = C1Battery) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 180, origin = {-37, 50})));
      wbEHPTlib.SupportModels.ConnectorRelated.Conn conn annotation(
        Placement(transformation(extent = {{-20, -20}, {20, 20}}, rotation = 90, origin = {-100, -2})));
      Modelica.Blocks.Sources.RealExpression SOCs(y = SOC) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {-80, 30})));
      Modelica.Blocks.Sources.RealExpression outPow(y = (p.v - n.v) * n.i) annotation(
        Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = -90, origin = {-80, -26})));
      Modelica.Electrical.Analog.Basic.Resistor R1(final R = R1Battery) annotation(
        Placement(visible = true, transformation(origin = {-37, 72}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
    protected
      parameter Real k = ((1 - efficiency) * (eBattMax + eBattMin) - 2 * (1 + efficiency) * Rtot * iCellEfficiency) / ((1 + efficiency) * (eBattMax + eBattMin) - 2 * (1 - efficiency) * Rtot * iCellEfficiency);
      parameter Real efficiencyMax = (eBattMin + eBattMax - 2 * Rtot * iCellEfficiency) / (eBattMin + eBattMax + 2 * Rtot * iCellEfficiency);
      final parameter Modelica.SIunits.Capacitance C = QCellNom / (ECellMax - ECellMin) "Cell capacitance";
      parameter Modelica.SIunits.Current IBatteryMax = ICellMax * np "Maximum battery current";
      parameter Modelica.SIunits.Voltage eBattMin = ECellMin * ns "Minimum battery voltage";
      parameter Modelica.SIunits.Voltage eBattMax = ECellMax * ns "Maximum battery voltage";
      parameter Modelica.SIunits.ElectricCharge QBatteryNominal = QCellNom * np "Battery admissible electric charge";
      parameter Modelica.SIunits.Capacitance CBattery = C * np / ns "Battery capacitance";
      parameter Modelica.SIunits.Resistance R0Battery = R0Cell * ns / np "Series inner resistance R0 of cell package";
      parameter Modelica.SIunits.Resistance R1Battery = R1Cell * ns / np "Series inner resistance R1 of cell package";
      parameter Modelica.SIunits.Resistance Rtot = R0Battery + R1Battery;
      parameter Modelica.SIunits.Capacitance C1Battery = C1Cell * np / ns "Battery series inner capacitance C1";
      Modelica.SIunits.Voltage ECell "Cell e.m.f.";
      Modelica.SIunits.Current iCellStray "Cell stray current";
      Modelica.SIunits.Voltage eBatt(start = eBattMin + SOCInit * (eBattMax - eBattMin), fixed = true) "Battery e.m.f.";
      Modelica.SIunits.Current iBatteryStray "Cell parasitic current";
      Modelica.Electrical.Analog.Sensors.CurrentSensor currentSensor annotation(
        Placement(transformation(extent = {{70, 50}, {90, 70}}, rotation = 0)));
      Modelica.Blocks.Math.Gain gain(k = k) annotation(
        Placement(transformation(origin = {60, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
      Modelica.Blocks.Math.Abs abs1 annotation(
        Placement(transformation(extent = {{34, -10}, {14, 10}}, rotation = 0)));
    equation
      connect(R1.p, R0.n) annotation(
        Line(points = {{-27, 72}, {-18, 72}, {-18, 60}, {10, 60}}, color = {0, 0, 255}));
      connect(R1.p, C1.p) annotation(
        Line(points = {{-27, 72}, {-18, 72}, {-18, 50}, {-27, 50}}, color = {0, 0, 255}));
      connect(R1.n, cBattery.p) annotation(
        Line(points = {{-47, 72}, {-60, 72}, {-60, 10}}, color = {0, 0, 255}));
      assert(SOCMin >= 0, "SOCMin must be greater than, or equal to 0");
      assert(SOCMax <= 1, "SOCMax must be smaller than, or equal to 1");
      assert(efficiency <= efficiencyMax, "Overall charging/discharging energy efficiency is too big with respect to the actual serial resistance (EfficiencyMax =" + String(efficiencyMax) + ")");
      assert(SOCMin < SOCMax, "SOCMax(=" + String(SOCMax) + ") must be greater than SOCMin(=" + String(SOCMin) + ")");
      assert(SOCInit >= SOCMin, "SOCInit(=" + String(SOCInit) + ") must be greater than, or equal to SOCMin(=" + String(SOCMin) + ")");
      assert(SOCInit <= SOCMax, "SOCInit(=" + String(SOCInit) + ") must be smaller than, or equal to SOCMax(=" + String(SOCMax) + ")");
      iBatteryStray = Ip.i;
      iCellStray = iBatteryStray / np;
      eBatt = cBattery.v;
//Solo per dare maggiore chiarezza all'utente con un nome significativo
      ECell = eBatt / ns;
      powDeliv = (p.v - n.v) * n.i;
      assert(abs(p.i / np) < ICellMax, "Battery cell current i=" + String(abs(p.i / np)) + "\n exceeds max admissable ICellMax (=" + String(ICellMax) + "A)");
      SOC = (eBatt - eBattMin) / (eBattMax - eBattMin);
//*(SOCMax-SOCMin)+SOCMin);
      assert(SOC <= SOCMax, "Battery is fully charged: State of charge reached maximum limit (=" + String(SOCMax) + ")");
      assert(SOCMin <= SOC, "Battery is fully discharged: State of charge reached minimum limit (=" + String(SOCMin) + ")");
      connect(R0.p, currentSensor.p) annotation(
        Line(points = {{30, 60}, {70, 60}}, color = {0, 0, 255}));
      connect(Ip.p, R0.n) annotation(
        Line(points = {{-6, 10}, {-6, 60}, {10, 60}}, color = {0, 0, 255}));
      connect(currentSensor.i, gain.u) annotation(
        Line(points = {{80, 50}, {80, -1.46958e-015}, {72, -1.46958e-015}}, color = {0, 0, 127}));
      connect(abs1.u, gain.y) annotation(
        Line(points = {{36, 0}, {39.5, 0}, {39.5, 1.33227e-015}, {49, 1.33227e-015}}, color = {0, 0, 127}));
      connect(abs1.y, Ip.i) annotation(
        Line(points = {{13, 0}, {7, 0}, {7, -1.28588e-015}, {1, -1.28588e-015}}, color = {0, 0, 127}));
      connect(currentSensor.n, p) annotation(
        Line(points = {{90, 60}, {90, 60}, {100, 60}}, color = {0, 0, 255}));
      connect(Ip.n, n) annotation(
        Line(points = {{-6, -10}, {-6, -60}, {100, -60}}, color = {0, 0, 255}));
      connect(n, cBattery.n) annotation(
        Line(points = {{100, -60}, {-60, -60}, {-60, -10}}, color = {0, 0, 255}));
      connect(C1.n, cBattery.p) annotation(
        Line(points = {{-47, 50}, {-60, 50}, {-60, 10}}, color = {0, 0, 255}, smooth = Smooth.None));
      connect(conn.batSOC, SOCs.y) annotation(
        Line(points = {{-100, -2}, {-100, 8.5}, {-80, 8.5}, {-80, 19}}, color = {255, 204, 51}, thickness = 0.5, smooth = Smooth.None),
        Text(string = "%first", index = -1, extent = {{-6, 3}, {-6, 3}}));
      connect(outPow.y, conn.batPowDel) annotation(
        Line(points = {{-80, -15}, {-80, -2}, {-100, -2}}, color = {0, 0, 127}, smooth = Smooth.None),
        Text(string = "%second", index = 1, extent = {{6, 3}, {6, 3}}));
      annotation(
        Documentation(info = "<html>
<p>Battery model with non-unity coulombic efficiency. </p>
<p>The main cell branch contains an e.m.f. that is linearly increasing with SOC, simulated through an equivalent capacitor, the resistance R0 and a parallel R-C couple. </p>
<p>The full battery is composed by np rows in parallel, each of them containing ns cells in series.</p>
<p>It interfaces with monitoring systems tohrough an expandable connector. Output signals:</p>
<p>- state-of-charge &QUOT;batSOC&QUOT;</p>
<p>- outputted power &QUOT;batPowDel&QUOT;.</p>
</html>", revisions = "<html><table border=\"1\" rules=\"groups\">
    <thead>
    <tr><td>Version</td>  <td>Date</td>  <td>Comment</td></tr>
    </thead>
    <tbody>
    <tr><td>1.0.0</td>  <td>2006-01-12</td>  <td> </td></tr>
    <tr><td>1.0.3</td>  <td>2006-08-31</td>  <td> Improved assert statements </td></tr>
    <tr><td>1.0.6</td>  <td>2007-05-14</td>  <td> The documentation changed slightly </td></tr>
    </tbody>
    </table>
    </html>"),
        Diagram(coordinateSystem(extent = {{-100, -80}, {100, 80}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics),
        Icon(coordinateSystem(extent = {{-100, -80}, {100, 80}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(lineColor = {95, 95, 95}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-100, 80}, {80, -82}}), Line(points = {{-92, 6}, {-52, 6}}, color = {0, 0, 255}), Rectangle(lineColor = {0, 0, 255}, fillColor = {0, 0, 255}, fillPattern = FillPattern.Solid, extent = {{-82, -3}, {-65, -10}}), Line(points = {{-73, 63}, {98, 64}}, color = {0, 0, 255}), Rectangle(lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{38, 69}, {68, 57}}), Rectangle(lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, extent = {{-37.5, 68}, {-6.5, 56}}), Line(points = {{-19.5, 49}, {-19.5, 32}}, color = {0, 0, 255}), Line(points = {{-54.5, 63}, {-54.5, 41}, {-25.5, 41}}, color = {0, 0, 255}), Line(points = {{9.5, 62}, {9.5, 40}, {-19.5, 40}}, color = {0, 0, 255}), Line(points = {{-73, 63}, {-73, 5}}, color = {0, 0, 255}), Line(points = {{-73, -6}, {-73, -60}, {96, -60}}, color = {0, 0, 255}), Line(points = {{26, 63}, {26, -61}}, color = {0, 0, 255}), Line(points = {{-25.5, 49}, {-25.5, 32}}, color = {0, 0, 255}), Polygon(lineColor = {0, 0, 255}, fillColor = {255, 255, 255}, fillPattern = FillPattern.Solid, points = {{26, 22}, {14, 4}, {26, -14}, {38, 4}, {26, 22}}), Line(points = {{20, 4}, {32, 4}}, color = {0, 0, 255}), Polygon(lineColor = {0, 0, 255}, points = {{22, -20}, {30, -20}, {26, -32}, {22, -20}}), Text(origin = {-4, -22}, lineColor = {0, 0, 255}, extent = {{-100, 150}, {100, 110}}, textString = "%name")}));
    end Batt1Conn;
    annotation(
      Icon(graphics = {Rectangle(lineColor = {200, 200, 200}, fillColor = {248, 248, 248}, fillPattern = FillPattern.HorizontalCylinder, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0), Rectangle(lineColor = {128, 128, 128}, extent = {{-100.0, -100.0}, {100.0, 100.0}}, radius = 25.0), Ellipse(extent = {{-36, 40}, {40, -36}}, lineColor = {0, 0, 0}), Line(points = {{4, 82}, {-6, 82}, {-10, 72}, {-24, 68}, {-34, 78}, {-46, 70}, {-42, 58}, {-54, 46}, {-66, 50}, {-74, 36}, {-66, 30}, {-68, 16}, {-78, 12}, {-78, 2}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{4, -78}, {-6, -78}, {-10, -68}, {-24, -64}, {-34, -74}, {-46, -66}, {-42, -54}, {-54, -42}, {-66, -46}, {-74, -32}, {-66, -26}, {-68, -12}, {-78, -8}, {-78, 2}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{2, -78}, {12, -78}, {16, -68}, {30, -64}, {40, -74}, {52, -66}, {48, -54}, {60, -42}, {72, -46}, {80, -32}, {72, -26}, {74, -12}, {84, -8}, {84, 2}}, color = {0, 0, 0}, smooth = Smooth.None), Line(points = {{2, 82}, {12, 82}, {16, 72}, {30, 68}, {40, 78}, {52, 70}, {48, 58}, {60, 46}, {72, 50}, {80, 36}, {72, 30}, {74, 16}, {84, 12}, {84, 2}}, color = {0, 0, 0}, smooth = Smooth.None)}));
  end SupportModels;

  package EV
    model FirstEV "Simulates a very basic Electric Vehicle"
      import Modelica;
      extends Modelica.Icons.Example;
      SupportModels.DragForce dragF(Cx = 0.26, rho = 1.226, S = 2.2, fc = 0.014, m = mass.m) annotation(
        Placement(visible = true, transformation(origin = {100, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
      Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = 6) annotation(
        Placement(visible = true, transformation(origin = {-8, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Torque torque annotation(
        Placement(visible = true, transformation(extent = {{-76, 10}, {-56, 30}}, rotation = 0)));
      SupportModels.PropDriver driver(CycleFileName = "sort1.txt", extrapolation = Modelica.Blocks.Types.Extrapolation.Periodic, k = 1000, yMax = 100000.0) annotation(
        Placement(visible = true, transformation(extent = {{-112, 10}, {-92, 30}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass mass(m = 1300) annotation(
        Placement(visible = true, transformation(extent = {{42, 10}, {62, 30}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor velSens annotation(
        Placement(visible = true, transformation(origin = {74, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
      Modelica.Mechanics.Rotational.Components.IdealRollingWheel wheel(radius = 0.5715) annotation(
        Placement(visible = true, transformation(extent = {{10, 10}, {30, 30}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia(J = 5) annotation(
        Placement(visible = true, transformation(extent = {{-48, 10}, {-28, 30}}, rotation = 0)));
    equation
      connect(inertia.flange_b, gear.flange_a) annotation(
        Line(points = {{-28, 20}, {-18, 20}}));
      connect(inertia.flange_a, torque.flange) annotation(
        Line(points = {{-48, 20}, {-52, 20}, {-56, 20}}));
      connect(mass.flange_a, wheel.flangeT) annotation(
        Line(points = {{42, 20}, {30, 20}}, color = {0, 127, 0}));
      connect(gear.flange_b, wheel.flangeR) annotation(
        Line(points = {{2, 20}, {2, 20}, {10, 20}}));
      connect(velSens.flange, mass.flange_b) annotation(
        Line(points = {{74, 4.44089e-16}, {74, 20}, {62, 20}}, color = {0, 127, 0}));
      connect(driver.V, velSens.v) annotation(
        Line(points = {{-102, 8.8}, {-102, -36}, {74, -36}, {74, -21}}, color = {0, 0, 127}));
      connect(dragF.flange, mass.flange_b) annotation(
        Line(points = {{90, 20}, {62, 20}}, color = {0, 127, 0}));
      connect(torque.tau, driver.tauRef) annotation(
        Line(points = {{-78, 20}, {-91, 20}}, color = {0, 0, 127}));
      annotation(
        experimentSetupOutput(derivatives = false),
        Documentation(info = "<html>
<p>Very basic introductory EV model</p>
</html>"),
        Commands,
        Diagram(coordinateSystem(extent = {{-120, -40}, {120, 40}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(extent = {{-84, 36}, {-24, 4}}, lineColor = {28, 108, 200}, pattern = LinePattern.Dash), Text(extent = {{-82, 2}, {-26, -4}}, lineColor = {28, 108, 200}, pattern = LinePattern.Dash, textString = "electric drive")}),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        experiment(StartTime = 0, StopTime = 200, Tolerance = 0.0001, Interval = 0.1),
        __OpenModelica_simulationFlags(jacobian = "", s = "dassl", lv = "LOG_STATS"));
    end FirstEV;

    model FirstEVpow "Simulates a very basic Electric Vehicle"
      import Modelica;
      extends Modelica.Icons.Example;
      SupportModels.DragForce dragF(Cx = 0.26, rho = 1.226, S = 2.2, fc = 0.014, m = mass.m) annotation(
        Placement(visible = true, transformation(origin = {100, -22}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
      Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = 6) annotation(
        Placement(visible = true, transformation(origin = {-8, 20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sources.Torque torque annotation(
        Placement(visible = true, transformation(extent = {{-76, 10}, {-56, 30}}, rotation = 0)));
      SupportModels.PropDriver driver(CycleFileName = "sort1.txt", extrapolation = Modelica.Blocks.Types.Extrapolation.Periodic, k = 1000, yMax = 100000.0) annotation(
        Placement(visible = true, transformation(extent = {{-112, 10}, {-92, 30}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass mass(m = 1300) annotation(
        Placement(visible = true, transformation(extent = {{54, 10}, {74, 30}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor velSens annotation(
        Placement(visible = true, transformation(origin = {78, -10}, extent = {{-10, -10}, {10, 10}}, rotation = 270)));
      Modelica.Mechanics.Rotational.Components.IdealRollingWheel wheel(radius = 0.5715) annotation(
        Placement(visible = true, transformation(extent = {{10, 10}, {30, 30}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.Inertia inertia(J = 5) annotation(
        Placement(visible = true, transformation(extent = {{-48, 10}, {-28, 30}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sensors.PowerSensor pP2 annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = -90, origin = {100, 6})));
      Modelica.Mechanics.Translational.Sensors.PowerSensor pP1 annotation(
        Placement(transformation(extent = {{36, 12}, {52, 28}})));
    equation
      connect(inertia.flange_b, gear.flange_a) annotation(
        Line(points = {{-28, 20}, {-18, 20}}));
      connect(inertia.flange_a, torque.flange) annotation(
        Line(points = {{-48, 20}, {-52, 20}, {-56, 20}}));
      connect(gear.flange_b, wheel.flangeR) annotation(
        Line(points = {{2, 20}, {2, 20}, {10, 20}}));
      connect(velSens.flange, mass.flange_b) annotation(
        Line(points = {{78, 4.44089e-016}, {78, 20}, {74, 20}}, color = {0, 127, 0}));
      connect(driver.V, velSens.v) annotation(
        Line(points = {{-102, 8.8}, {-102, -36}, {78, -36}, {78, -21}}, color = {0, 0, 127}));
      connect(torque.tau, driver.tauRef) annotation(
        Line(points = {{-78, 20}, {-91, 20}}, color = {0, 0, 127}));
      connect(pP2.flange_b, dragF.flange) annotation(
        Line(points = {{100, -4}, {100, -12}}, color = {0, 127, 0}));
      connect(pP2.flange_a, mass.flange_b) annotation(
        Line(points = {{100, 16}, {100, 20}, {74, 20}}, color = {0, 127, 0}));
      connect(mass.flange_a, pP1.flange_b) annotation(
        Line(points = {{54, 20}, {54, 20}, {52, 20}}, color = {0, 127, 0}));
      connect(wheel.flangeT, pP1.flange_a) annotation(
        Line(points = {{30, 20}, {34, 20}, {36, 20}}, color = {0, 127, 0}));
      annotation(
        experimentSetupOutput(derivatives = false),
        Documentation(info = "<html>
<p>Very basic introductory EV model with power measurements.</p>
</html>"),
        Commands,
        Diagram(coordinateSystem(extent = {{-120, -40}, {120, 40}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(extent = {{-84, 36}, {-24, 4}}, lineColor = {28, 108, 200}, pattern = LinePattern.Dash), Text(extent = {{-82, 2}, {-26, -4}}, lineColor = {28, 108, 200}, pattern = LinePattern.Dash, textString = "electric drive")}),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        experiment(StartTime = 0, StopTime = 200, Tolerance = 0.0001, Interval = 0.1),
        __OpenModelica_simulationFlags(jacobian = "", s = "dassl", lv = "LOG_STATS"));
    end FirstEVpow;

    model MBEV "Simulates a very basic Electric Vehicle"
      import Modelica;
      extends Modelica.Icons.Example;
      Modelica.SIunits.Energy enBatDel, enDTdel, enP1del, enBattLoss, enBraking;
      Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = 6) annotation(
        Placement(visible = true, transformation(origin = {-20, 14}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      SupportModels.PropDriver driver(CycleFileName = "NEDC.txt", extrapolation = Modelica.Blocks.Types.Extrapolation.Periodic, yMax = 100000.0, k = 100) annotation(
        Placement(visible = true, transformation(extent = {{-116, -10}, {-96, 10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.IdealRollingWheel wheel(radius = 0.5715) annotation(
        Placement(visible = true, transformation(extent = {{-4, 4}, {16, 24}}, rotation = 0)));
      wbEHPTlib.MapBased.OneFlange eleDrive(J = 0.25, effMapName = "effTable", mapsFileName = "EVmaps.txt", tauMax = 150, wMax = 500) "Electric Drive" annotation(
        Placement(visible = true, transformation(extent = {{-76, 6}, {-56, 24}}, rotation = 0)));
      SupportModels.Batt1 batt1(SOCInit = 0.7, QCellNom = 100 * 3600, ns = 100) annotation(
        Placement(transformation(extent = {{-112, 34}, {-92, 54}})));
      Modelica.Electrical.Analog.Basic.Ground ground annotation(
        Placement(transformation(extent = {{-98, 12}, {-78, 32}})));
      Modelica.Mechanics.Translational.Sensors.PowerSensor pP1 annotation(
        Placement(visible = true, transformation(origin = {32, 14}, extent = {{-6, -8}, {6, 8}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass mass(m = 1300) annotation(
        Placement(visible = true, transformation(extent = {{56, 4}, {76, 24}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sensors.PowerSensor pP2 annotation(
        Placement(visible = true, transformation(origin = {98, 4}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor velSens annotation(
        Placement(visible = true, transformation(origin = {68, -42}, extent = {{-10, -10}, {10, 10}}, rotation = 180)));
      SupportModels.DragForce dragF(Cx = 0.26, rho = 1.226, S = 2.2, fc = 0.014, m = mass.m) annotation(
        Placement(visible = true, transformation(origin = {98, -24}, extent = {{-10, -10}, {10, 10}}, rotation = 90)));
      Modelica.Mechanics.Translational.Sources.Force brake annotation(
        Placement(visible = true, transformation(origin = {32, -20}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.TorqueSensor edTau annotation(
        Placement(transformation(extent = {{-8, -8}, {8, 8}}, rotation = 0, origin = {-42, 14})));
      Modelica.Blocks.Math.Add add(k1 = -1) annotation(
        Placement(transformation(extent = {{-42, -26}, {-30, -14}})));
      Modelica.Blocks.Math.Gain tqToForce(k = gear.ratio * wheel.radius) annotation(
        Placement(visible = true, transformation(extent = {{0, -26}, {12, -14}}, rotation = 0)));
      Modelica.Blocks.Nonlinear.Limiter cutNeg(limitsAtInit = true, uMax = 0, uMin = -Modelica.Constants.inf) annotation(
        Placement(visible = true, transformation(origin = {-14, -20}, extent = {{-6, -6}, {6, 6}}, rotation = 0)));
    equation
      connect(tqToForce.y, brake.f) annotation(
        Line(points = {{12.6, -20}, {18, -20}, {18, -20}, {20, -20}}, color = {0, 0, 127}));
      connect(cutNeg.y, tqToForce.u) annotation(
        Line(points = {{-7.4, -20}, {-2, -20}, {-2, -20}, {-1.2, -20}}, color = {0, 0, 127}));
      connect(cutNeg.u, add.y) annotation(
        Line(points = {{-21.2, -20}, {-30, -20}, {-30, -20}, {-29.4, -20}}, color = {0, 0, 127}));
      connect(edTau.flange_a, eleDrive.flange_a) annotation(
        Line(points = {{-50, 14}, {-53, 14}, {-56, 14}}));
      connect(ground.p, eleDrive.pin_n) annotation(
        Line(points = {{-88, 32}, {-88, 38}, {-71.8, 38}, {-71.8, 24}}, color = {0, 0, 255}));
      connect(batt1.p, eleDrive.pin_p) annotation(
        Line(points = {{-92, 50}, {-62, 50}, {-62, 24}}, color = {0, 0, 255}));
      connect(batt1.n, eleDrive.pin_n) annotation(
        Line(points = {{-91.9, 38}, {-71.8, 38}, {-71.8, 24}}, color = {0, 0, 255}));
      connect(eleDrive.tauRef, driver.tauRef) annotation(
        Line(points = {{-77.4, 14}, {-86, 14}, {-86, 0}, {-95, 0}}, color = {0, 0, 127}));
      connect(brake.flange, pP1.flange_b) annotation(
        Line(points = {{42, -20}, {46, -20}, {46, 14}, {38, 14}}, color = {0, 127, 0}));
      connect(velSens.flange, pP2.flange_a) annotation(
        Line(points = {{78, -42}, {78, 14}, {98, 14}}, color = {0, 127, 0}));
      connect(driver.V, velSens.v) annotation(
        Line(points = {{-106, -11.2}, {-106, -42}, {57, -42}}, color = {0, 0, 127}));
      connect(mass.flange_a, pP1.flange_b) annotation(
        Line(points = {{56, 14}, {38, 14}}, color = {0, 127, 0}));
      connect(pP2.flange_a, mass.flange_b) annotation(
        Line(points = {{98, 14}, {76, 14}}, color = {0, 127, 0}));
      connect(pP1.flange_a, wheel.flangeT) annotation(
        Line(points = {{26, 14}, {16, 14}}, color = {0, 127, 0}));
      connect(gear.flange_b, wheel.flangeR) annotation(
        Line(points = {{-10, 14}, {-4, 14}}));
      connect(dragF.flange, pP2.flange_b) annotation(
        Line(points = {{98, -14}, {98, -6}}, color = {0, 127, 0}));
      der(enBatDel) = (batt1.p.v - batt1.n.v) * batt1.n.i;
      der(enDTdel) = eleDrive.powSensor.power;
      der(enP1del) = pP1.power;
      der(enBattLoss) = batt1.powerLoss;
      der(enBraking) = if pP1.power > 0 then 0 else -pP1.power;
      connect(add.u2, driver.tauRef) annotation(
        Line(points = {{-43.2, -23.6}, {-86, -23.6}, {-86, 0}, {-95, 0}}, color = {0, 0, 127}));
      connect(edTau.flange_b, gear.flange_a) annotation(
        Line(points = {{-34, 14}, {-30, 14}, {-30, 14}}, color = {0, 0, 0}));
      connect(add.u1, edTau.tau) annotation(
        Line(points = {{-43.2, -16.4}, {-48.4, -16.4}, {-48.4, 5.2}}, color = {0, 0, 127}));
      annotation(
        experimentSetupOutput(derivatives = false),
        Documentation(info = "<html>
<p>Simple map-based EV model with battery.</p>
</html>"),
        Commands,
        Diagram(coordinateSystem(extent = {{-120, -60}, {120, 60}}, preserveAspectRatio = false)),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        experiment(StartTime = 0, StopTime = 1400, Tolerance = 0.0001, Interval = 0.1),
        __OpenModelica_commandLineOptions = "");
    end MBEV;
  end EV;

  package sHEV
    model SHEVpowerFilt "Ice, Generator, DriveTrain, all map-based"
      //€
      extends Modelica.Icons.Example;
      Modelica.Electrical.Analog.Basic.Ground ground1 annotation(
        Placement(transformation(extent = {{-8, -8}, {8, 8}}, rotation = 0, origin = {-42, -2})));
      Modelica.Mechanics.Rotational.Components.IdealRollingWheel wheel(radius = 0.473) annotation(
        Placement(transformation(extent = {{-52, -46}, {-38, -32}})));
      Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = 10) annotation(
        Placement(transformation(extent = {{-78, -46}, {-64, -32}})));
      Modelica.Mechanics.Translational.Components.Mass mass(m = 14000) annotation(
        Placement(transformation(extent = {{-8, -48}, {10, -30}})));
      Modelica.Mechanics.Translational.Sensors.PowerSensor powProp annotation(
        Placement(transformation(extent = {{-7, -7}, {7, 7}}, rotation = 0, origin = {-23, -39})));
      Modelica.Mechanics.Translational.Sensors.PowerSensor powDrag annotation(
        Placement(transformation(extent = {{-7, -7}, {7, 7}}, rotation = 0, origin = {45, -39})));
      SupportModels.DragForce dragForce(S = 6.5, fc = 0.01, Cx = 0.65, m = mass.m) annotation(
        Placement(transformation(extent = {{-8, -8}, {8, 8}}, rotation = 90, origin = {80, -48})));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor speedSensor1 annotation(
        Placement(transformation(extent = {{-8, -8}, {8, 8}}, rotation = 270, origin = {26, -54})));
      Modelica.Blocks.Continuous.FirstOrder powFilt(y_start = 20e3, T = 500) annotation(
        Placement(visible = true, transformation(extent = {{12, 64}, {-4, 80}}, rotation = 0)));
      Modelica.Blocks.Nonlinear.Limiter limiter(uMax = 100e3, uMin = 0) annotation(
        Placement(visible = true, transformation(extent = {{-14, 64}, {-30, 80}}, rotation = 0)));
      SupportModels.PropDriver driver(CycleFileName = "nedc.txt", extrapolation = Modelica.Blocks.Types.Extrapolation.Periodic, k = 200.0, yMax = 2e3) annotation(
        Placement(visible = true, transformation(extent = {{-98, 76}, {-78, 96}}, rotation = 0)));
      wbEHPTlib.MapBased.Genset genset(OptiTime = 1, mapsFileName = "SHEVmaps.txt", wGenMax = 300) annotation(
        Placement(transformation(extent = {{-80, 8}, {-50, 38}})));
      SupportModels.Batt1 battery(QCellNom = 200 * 3600, ICellMax = 1000, R0Cell = 0.35E-3, efficiency = 0.9, iCellEfficiency = 200, ns = 200) annotation(
        Placement(transformation(extent = {{0, 18}, {20, 38}})));
      wbEHPTlib.MapBased.OneFlange drive(tauMax = 1000, powMax = 150e3, wMax = 3000, mapsFileName = "SHEVmaps.txt", effMapName = "motEffTable") annotation(
        Placement(visible = true, transformation(extent = {{40, 42}, {60, 22}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.PowerSensor drivePow annotation(
        Placement(transformation(extent = {{66, 40}, {86, 20}})));
      Modelica.Electrical.Analog.Sensors.PowerSensor gsPow annotation(
        Placement(transformation(extent = {{-32, 22}, {-12, 42}})));
    equation
      connect(drive.pin_n, battery.p) annotation(
        Line(points = {{44.2, 22}, {44.1, 22}, {44.1, 20}, {28, 20}, {28, 34}, {20, 34}}, color = {0, 0, 255}));
      connect(gsPow.nv, drive.pin_p) annotation(
        Line(points = {{-22, 22}, {-22, 14}, {54, 14}, {54, 22}}, color = {0, 0, 255}));
      connect(drivePow.flange_a, drive.flange_a) annotation(
        Line(points = {{66, 30}, {61, 30}, {61, 33.1111}, {60, 33.1111}}));
      connect(battery.n, drive.pin_p) annotation(
        Line(points = {{20.1, 22}, {24, 22}, {24, 14}, {54, 14}, {54, 22}}, color = {0, 0, 255}));
      connect(genset.pin_n, drive.pin_p) annotation(
        Line(points = {{-49.7, 14}, {54, 14}, {54, 22}}, color = {0, 0, 255}));
      connect(drive.tauRef, driver.tauRef) annotation(
        Line(points = {{38.6, 33.1111}, {38.6, 34}, {32, 34}, {32, 86}, {-77, 86}}, color = {0, 0, 127}));
      connect(drivePow.power, powFilt.u) annotation(
        Line(points = {{68, 41}, {68, 72}, {13.6, 72}}, color = {0, 0, 127}));
      connect(powFilt.y, limiter.u) annotation(
        Line(points = {{-4.8, 72}, {-12.4, 72}}, color = {0, 0, 127}));
      connect(limiter.y, genset.powRef) annotation(
        Line(points = {{-30.8, 72}, {-55.85, 72}, {-55.85, 40.25}}, color = {0, 0, 127}));
      connect(speedSensor1.v, driver.V) annotation(
        Line(points = {{26, -62.8}, {26, -74}, {-94, -74}, {-94, 68}, {-88, 68}, {-88, 74.8}}, color = {0, 0, 127}));
      connect(gear.flange_b, wheel.flangeR) annotation(
        Line(points = {{-64, -39}, {-52, -39}}, color = {0, 0, 0}, smooth = Smooth.None));
      connect(mass.flange_a, powProp.flange_b) annotation(
        Line(points = {{-8, -39}, {-16, -39}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(powProp.flange_a, wheel.flangeT) annotation(
        Line(points = {{-30, -39}, {-38, -39}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(powDrag.flange_a, mass.flange_b) annotation(
        Line(points = {{38, -39}, {10, -39}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(powDrag.flange_b, dragForce.flange) annotation(
        Line(points = {{52, -39}, {66, -39}, {66, -40}, {80, -40}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(speedSensor1.flange, mass.flange_b) annotation(
        Line(points = {{26, -46}, {26, -39}, {10, -39}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(genset.pin_n, ground1.p) annotation(
        Line(points = {{-49.7, 14}, {-42, 14}, {-42, 6}}, color = {0, 0, 255}, smooth = Smooth.None));
      connect(drivePow.flange_b, gear.flange_a) annotation(
        Line(points = {{86, 30}, {90, 30}, {90, -22}, {-86, -22}, {-86, -39}, {-78, -39}}, color = {0, 0, 0}));
      connect(gsPow.pc, genset.pin_p) annotation(
        Line(points = {{-32, 32}, {-36, 32}, {-50, 32}}, color = {0, 0, 255}));
      connect(gsPow.nc, battery.p) annotation(
        Line(points = {{-12, 32}, {-8, 32}, {-8, 48}, {28, 48}, {28, 34}, {20, 34}}, color = {0, 0, 255}));
      connect(gsPow.pv, gsPow.pc) annotation(
        Line(points = {{-22, 42}, {-32, 42}, {-32, 32}}, color = {0, 0, 255}));
      annotation(
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -80}, {100, 100}}), graphics = {Rectangle(extent = {{-90, -28}, {94, -70}}, lineColor = {255, 0, 0}, pattern = LinePattern.Dash), Rectangle(extent = {{-90, 52}, {96, -10}}, lineColor = {255, 0, 0}, pattern = LinePattern.Dash), Rectangle(extent = {{-60, 96}, {94, 58}}, lineColor = {255, 0, 0}, pattern = LinePattern.Dash), Text(extent = {{68, 74}, {94, 66}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "EMS"), Text(extent = {{-96, -60}, {-44, -68}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "MechProp"), Text(extent = {{12, 0}, {58, -8}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "PowerTrain")}),
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics),
        experiment(StopTime = 1500, StartTime = 0, Tolerance = 1e-06, Interval = 0.75),
        experimentSetupOutput(derivatives = false),
        Documentation(info = "<html>
<p>This is a SHEV model which has an Energy Management System able to control the power flow with basic logic: requests the ICE to deliver the average load power.</p>
</html>"));
    end SHEVpowerFilt;

    model SHEVpowerFiltSoc "Ice, Generator, DriveTrain, all map-based"
      //€
      extends Modelica.Icons.Example;
      Modelica.Electrical.Analog.Basic.Ground ground1 annotation(
        Placement(transformation(extent = {{-8, 8}, {8, -8}}, rotation = 0, origin = {-42, -4})));
      Modelica.Mechanics.Rotational.Components.IdealRollingWheel wheel(radius = 0.473) annotation(
        Placement(transformation(extent = {{-52, -50}, {-40, -38}})));
      Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = 10) annotation(
        Placement(transformation(extent = {{-78, -50}, {-66, -38}})));
      Modelica.Mechanics.Translational.Components.Mass mass(m = 14000) annotation(
        Placement(transformation(extent = {{-8, -52}, {10, -34}})));
      Modelica.Mechanics.Translational.Sensors.PowerSensor powProp annotation(
        Placement(transformation(extent = {{-6, -6}, {6, 6}}, rotation = 0, origin = {-24, -44})));
      Modelica.Mechanics.Translational.Sensors.PowerSensor powDrag annotation(
        Placement(transformation(extent = {{-7, -7}, {7, 7}}, rotation = 0, origin = {45, -43})));
      SupportModels.DragForce dragForce(S = 6.5, fc = 0.01, Cx = 0.65, m = mass.m) annotation(
        Placement(transformation(extent = {{-8, -8}, {8, 8}}, rotation = 90, origin = {80, -52})));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor speedSensor1 annotation(
        Placement(transformation(extent = {{-8, -8}, {8, 8}}, rotation = 270, origin = {26, -58})));
      wbEVPkg.SupportModels.PropDriver driver(extrapolation = Modelica.Blocks.Types.Extrapolation.Periodic, k = 200.0, yMax = 2e3, CycleFileName = "nedc.txt") annotation(
        Placement(visible = true, transformation(extent = {{-100, 94}, {-80, 114}}, rotation = 0)));
      wbEHPTlib.MapBased.Genset genset(OptiTime = 1, wGenMax = 300, mapsFileName = "SHEVmaps.txt") annotation(
        Placement(transformation(extent = {{-80, -18}, {-50, 12}})));
      SupportModels.Batt1 battery(ICellMax = 1000, QCellNom = 200 * 3600, R0Cell = 0.35E-3, SOCInit = 0.6, efficiency = 0.9, iCellEfficiency = 200, ns = 200) annotation(
        Placement(transformation(extent = {{0, -8}, {20, 12}})));
      wbEHPTlib.MapBased.OneFlange2 drive(tauMax = 1000, powMax = 150e3, wMax = 3000, mapsFileName = "SHEVmaps.txt", effMapName = "motEffTable") annotation(
        Placement(visible = true, transformation(extent = {{40, 16}, {60, -4}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.PowerSensor drivePow annotation(
        Placement(transformation(extent = {{66, 14}, {86, -6}})));
      Modelica.Electrical.Analog.Sensors.PowerSensor gsPow annotation(
        Placement(transformation(extent = {{-32, -4}, {-12, 16}})));
      Modelica.Blocks.Math.Feedback fbSOC annotation(
        Placement(transformation(extent = {{20, 40}, {0, 60}})));
      Modelica.Blocks.Sources.Constant socRef_(k = 0.6) annotation(
        Placement(transformation(extent = {{-10, 10}, {10, -10}}, rotation = 180, origin = {38, 50})));
      Modelica.Blocks.Math.Gain socErrToPow(k = 3e6) annotation(
        Placement(transformation(extent = {{10, -10}, {-10, 10}}, rotation = 0, origin = {-24, 50})));
      Modelica.Blocks.Math.Add add annotation(
        Placement(visible = true, transformation(origin = {-6, 84}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
      Modelica.Blocks.Continuous.FirstOrder powFilt(y_start = 20e3, T = 500) annotation(
        Placement(visible = true, transformation(extent = {{34, 82}, {18, 98}}, rotation = 0)));
      Modelica.Blocks.Nonlinear.Limiter toPowRef(uMax = 100e3, uMin = 0) annotation(
        Placement(visible = true, transformation(extent = {{-30, 76}, {-46, 92}}, rotation = 0)));
    equation
      connect(gsPow.nv, drive.pin_p) annotation(
        Line(points = {{-22, -4}, {-22, -12}, {32, -12}, {32, -1}, {40, -1}, {40, -2}}, color = {0, 0, 255}));
      connect(battery.n, drive.pin_p) annotation(
        Line(points = {{20.1, -4}, {24, -4}, {24, -12}, {32, -12}, {32, -1}, {40, -1}, {40, -2}}, color = {0, 0, 255}));
      connect(genset.pin_n, drive.pin_p) annotation(
        Line(points = {{-49.7, -12}, {32, -12}, {32, -1}, {40, -1}, {40, -2}}, color = {0, 0, 255}));
      connect(drive.tauRef, driver.tauRef) annotation(
        Line(points = {{50, 15.5556}, {50, 104}, {-79, 104}}, color = {0, 0, 127}));
      connect(drive.pin_n, battery.p) annotation(
        Line(points = {{40, 11.5556}, {28, 11.5556}, {28, 8}, {20, 8}}, color = {0, 0, 255}));
      connect(drivePow.flange_a, drive.flange_a) annotation(
        Line(points = {{66, 4}, {61, 4}, {61, 5.11111}, {60, 5.11111}}));
      connect(socErrToPow.y, add.u2) annotation(
        Line(points = {{-35, 50}, {-46, 50}, {-46, 68}, {12, 68}, {12, 78}, {6, 78}}, color = {0, 0, 127}));
      connect(add.y, toPowRef.u) annotation(
        Line(points = {{-17, 84}, {-28.4, 84}}, color = {0, 0, 127}));
      connect(toPowRef.y, genset.powRef) annotation(
        Line(points = {{-46.8, 84}, {-56, 84}, {-56, 14.25}, {-55.85, 14.25}}, color = {0, 0, 127}));
      connect(powFilt.y, add.u1) annotation(
        Line(points = {{17.2, 90}, {6, 90}}, color = {0, 0, 127}));
      connect(drivePow.power, powFilt.u) annotation(
        Line(points = {{68, 15}, {68, 90}, {35.6, 90}}, color = {0, 0, 127}));
      connect(speedSensor1.v, driver.V) annotation(
        Line(points = {{26, -66.8}, {26, -78}, {-94, -78}, {-94, 44}, {-90, 44}, {-90, 92.8}}, color = {0, 0, 127}));
      connect(gear.flange_b, wheel.flangeR) annotation(
        Line(points = {{-66, -44}, {-52, -44}}, color = {0, 0, 0}, smooth = Smooth.None));
      connect(mass.flange_a, powProp.flange_b) annotation(
        Line(points = {{-8, -43}, {-12, -43}, {-12, -44}, {-18, -44}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(powProp.flange_a, wheel.flangeT) annotation(
        Line(points = {{-30, -44}, {-40, -44}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(powDrag.flange_a, mass.flange_b) annotation(
        Line(points = {{38, -43}, {10, -43}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(powDrag.flange_b, dragForce.flange) annotation(
        Line(points = {{52, -43}, {66, -43}, {66, -44}, {80, -44}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(speedSensor1.flange, mass.flange_b) annotation(
        Line(points = {{26, -50}, {26, -43}, {10, -43}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(genset.pin_n, ground1.p) annotation(
        Line(points = {{-49.7, -12}, {-42, -12}}, color = {0, 0, 255}, smooth = Smooth.None));
      connect(drivePow.flange_b, gear.flange_a) annotation(
        Line(points = {{86, 4}, {90, 4}, {90, -26}, {-86, -26}, {-86, -44}, {-78, -44}}, color = {0, 0, 0}));
      connect(gsPow.pc, genset.pin_p) annotation(
        Line(points = {{-32, 6}, {-36, 6}, {-50, 6}}, color = {0, 0, 255}));
      connect(gsPow.nc, battery.p) annotation(
        Line(points = {{-12, 6}, {-8, 6}, {-8, 22}, {28, 22}, {28, 8}, {20, 8}}, color = {0, 0, 255}));
      connect(gsPow.pv, gsPow.pc) annotation(
        Line(points = {{-22, 16}, {-32, 16}, {-32, 6}}, color = {0, 0, 255}));
      connect(battery.SOC, fbSOC.u2) annotation(
        Line(points = {{-1, 2}, {-4, 2}, {-4, 34}, {-4, 42}, {10, 42}}, color = {0, 0, 127}));
      connect(fbSOC.u1, socRef_.y) annotation(
        Line(points = {{18, 50}, {27, 50}}, color = {0, 0, 127}));
      connect(socErrToPow.u, fbSOC.y) annotation(
        Line(points = {{-12, 50}, {1, 50}}, color = {0, 0, 127}));
      annotation(
        Diagram(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -80}, {100, 120}}), graphics = {Rectangle(extent = {{-90, -32}, {94, -74}}, lineColor = {255, 0, 0}, pattern = LinePattern.Dash), Rectangle(extent = {{-90, 26}, {96, -22}}, lineColor = {255, 0, 0}, pattern = LinePattern.Dash), Rectangle(extent = {{-70, 114}, {96, 32}}, lineColor = {255, 0, 0}, pattern = LinePattern.Dash), Text(extent = {{68, 92}, {94, 84}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "EMS"), Text(extent = {{-96, -64}, {-44, -72}}, lineColor = {255, 0, 0}, fillColor = {0, 0, 0}, fillPattern = FillPattern.Solid, textString = "MechProp")}),
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics),
        experiment(StopTime = 2000, StartTime = 0, Tolerance = 1e-06, Interval = 4),
        experimentSetupOutput(derivatives = false),
        Documentation(info = "<html>
<p>This is a SHEV model which has an Energy Management System able to control the power flow with:</p>
<p>- basic logic: requests the ICE to deliver the average load power </p>
<p>- additional logic: SOC loop to avoid SOC drift.</p>
</html>"));
    end SHEVpowerFiltSoc;

    model SHEVpowerFiltSocOO "Ice, Generator, DriveTrain, all map-based"
      //€
      extends Modelica.Icons.Example;
      Modelica.Electrical.Analog.Basic.Ground ground1 annotation(
        Placement(visible = true, transformation(origin = {22, -12}, extent = {{-8, -8}, {8, 8}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.IdealRollingWheel wheel(radius = 0.473) annotation(
        Placement(visible = true, transformation(origin = {-48, -42}, extent = {{-8, -8}, {8, 8}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.IdealGear gear(ratio = 10) annotation(
        Placement(visible = true, transformation(extent = {{-82, -52}, {-62, -32}}, rotation = 0)));
      Modelica.Mechanics.Translational.Components.Mass mass(m = 14000) annotation(
        Placement(visible = true, transformation(extent = {{-2, -50}, {16, -32}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sensors.PowerSensor powProp annotation(
        Placement(visible = true, transformation(origin = {-21, -41}, extent = {{-9, -9}, {9, 9}}, rotation = 0)));
      Modelica.Mechanics.Translational.Sensors.PowerSensor powDrag annotation(
        Placement(visible = true, transformation(origin = {51, -41}, extent = {{-9, -9}, {9, 9}}, rotation = 0)));
      SupportModels.DragForce dragForce(S = 6.5, fc = 0.01, Cx = 0.65, m = mass.m) annotation(
        Placement(visible = true, transformation(origin = {76, -50}, extent = {{-8, -8}, {8, 8}}, rotation = 90)));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor speedSensor1 annotation(
        Placement(visible = true, transformation(origin = {26, -56}, extent = {{-8, -8}, {8, 8}}, rotation = 270)));
      SupportModels.Batt1 battery(ICellMax = 1000, R0Cell = 0.35E-3, efficiency = 0.9, iCellEfficiency = 200, ns = 200, SOCInit = 0.6, QCellNom = 50 * 3600) annotation(
        Placement(visible = true, transformation(origin = {-2, 26}, extent = {{-10, -10}, {10, 10}}, rotation = -90)));
      wbEHPTlib.MapBased.OneFlange2 drive(tauMax = 1000, powMax = 150e3, wMax = 3000, effMapName = "motEffTable", mapsFileName = "SHEVmaps.txt") annotation(
        Placement(visible = true, transformation(extent = {{36, 16}, {56, -4}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Sensors.PowerSensor drivePow annotation(
        Placement(visible = true, transformation(extent = {{62, 14}, {82, -6}}, rotation = 0)));
      Modelica.Electrical.Analog.Sensors.PowerSensor gsPow annotation(
        Placement(visible = true, transformation(extent = {{-36, 0}, {-16, 20}}, rotation = 0)));
      wbEHPTlib.MapBased.ECUs.EMS ems(powHigh = 60e3, powLow = 30e3, powMax = 200e3, powPerSoc = 300e3, socRef = 0.75) annotation(
        Placement(visible = true, transformation(origin = {-20, 58}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
      wbEHPTlib.MapBased.GensetOO genset(mapsFileName = "SHEVmaps.txt", tauMax = drive.tauMax) annotation(
        Placement(visible = true, transformation(extent = {{-84, -14}, {-54, 16}}, rotation = 0)));
      SupportModels.PropDriver driver(CycleFileName = "Sort1.txt", extrapolation = Modelica.Blocks.Types.Extrapolation.Periodic, k = 200.0, yMax = 2e3) annotation(
        Placement(visible = true, transformation(origin = {86, 60}, extent = {{10, -10}, {-10, 10}}, rotation = 0)));
    equation
      connect(battery.n, drive.pin_p) annotation(
        Line(points = {{-8, 15.9}, {-8, -8}, {10, -8}, {10, -1.55556}, {36, -1.55556}}, color = {0, 0, 255}));
      connect(gsPow.nv, drive.pin_p) annotation(
        Line(points = {{-26, 0}, {-26, -8}, {10, -8}, {10, -1.55556}, {36, -1.55556}}, color = {0, 0, 255}));
      connect(genset.pin_n, drive.pin_p) annotation(
        Line(points = {{-53.7, -8}, {-32, -8}, {10, -8}, {10, -1.55556}, {36, -1.55556}}, color = {0, 0, 255}));
      connect(ground1.p, drive.pin_p) annotation(
        Line(points = {{22, -4}, {22, -1.55556}, {36, -1.55556}}, color = {0, 0, 255}));
      connect(drivePow.flange_a, drive.flange_a) annotation(
        Line(points = {{62, 4}, {58, 4}, {58, 5.11111}, {56, 5.11111}}));
      connect(drive.pin_n, gsPow.nc) annotation(
        Line(points = {{36, 11.5556}, {22, 11.5556}, {22, 10}, {-16, 10}}, color = {0, 0, 255}));
      connect(driver.tauRef, drive.tauRef) annotation(
        Line(points = {{75, 60}, {46, 60}, {46, 15.5556}}, color = {0, 0, 127}));
      connect(speedSensor1.v, driver.V) annotation(
        Line(points = {{26, -64.8}, {26, -74}, {98, -74}, {98, 44}, {86, 44}, {86, 48.8}}, color = {0, 0, 127}));
      connect(gsPow.pc, genset.pin_p) annotation(
        Line(points = {{-36, 10}, {-54, 10}}, color = {0, 0, 255}));
      connect(ems.pcPowReq, genset.powRef) annotation(
        Line(points = {{-30.8, 52}, {-60, 52}, {-60, 18.4}}, color = {0, 0, 127}));
      connect(ems.on, genset.ON) annotation(
        Line(points = {{-30.8, 64}, {-78, 64}, {-78, 18.4}}, color = {255, 0, 255}));
      connect(gear.flange_b, wheel.flangeR) annotation(
        Line(points = {{-62, -42}, {-56, -42}}));
      connect(powProp.flange_a, wheel.flangeT) annotation(
        Line(points = {{-30, -41}, {-35, -41}, {-35, -42}, {-40, -42}}, color = {0, 127, 0}));
      connect(mass.flange_a, powProp.flange_b) annotation(
        Line(points = {{-2, -41}, {-12, -41}}, color = {0, 127, 0}));
      connect(battery.SOC, ems.soc) annotation(
        Line(points = {{-2, 37}, {-2, 52}, {-8, 52}}, color = {0, 0, 127}));
      connect(ems.edPow, drivePow.power) annotation(
        Line(points = {{-8, 64}, {38, 64}, {38, 64}, {64, 64}, {64, 15}}, color = {0, 0, 127}));
      connect(gsPow.pv, gsPow.pc) annotation(
        Line(points = {{-26, 20}, {-36, 20}, {-36, 10}}, color = {0, 0, 255}));
      connect(powDrag.flange_b, dragForce.flange) annotation(
        Line(points = {{60, -41}, {60, -42}, {76, -42}}, color = {0, 127, 0}));
      connect(drivePow.flange_b, gear.flange_a) annotation(
        Line(points = {{82, 4}, {88, 4}, {88, -18}, {-86, -18}, {-86, -42}, {-82, -42}}));
      connect(speedSensor1.flange, mass.flange_b) annotation(
        Line(points = {{26, -48}, {26, -41}, {16, -41}}, color = {0, 127, 0}));
      connect(powDrag.flange_a, mass.flange_b) annotation(
        Line(points = {{42, -41}, {42, -41}, {16, -41}}, color = {0, 127, 0}));
      connect(battery.p, gsPow.nc) annotation(
        Line(points = {{4, 16}, {4, 10}, {-16, 10}}, color = {0, 0, 255}));
      annotation(
        Diagram(coordinateSystem(extent = {{-100, -80}, {100, 80}}, initialScale = 0.1), graphics = {Rectangle(origin = {-2, 4}, lineColor = {255, 0, 0}, pattern = LinePattern.Dash, extent = {{-90, -32}, {94, -74}}), Rectangle(origin = {-4, 0}, lineColor = {255, 0, 0}, pattern = LinePattern.Dash, extent = {{-88, 40}, {96, -22}}), Text(origin = {0, 4}, lineColor = {255, 0, 0}, fillPattern = FillPattern.Solid, extent = {{-96, -64}, {-44, -72}}, textString = "MechProp")}),
        Icon(coordinateSystem(preserveAspectRatio = true, extent = {{-100, -100}, {100, 100}}), graphics),
        experiment(StopTime = 1800, __Dymola_NumberOfIntervals = 2000),
        experimentSetupOutput(derivatives = false),
        Documentation(info = "<html>
<p>This is a SHEV model which has an Energy Management System able to control the power flow with:</p>
<p>- basic logic: requests the ICE to deliver the average load power </p>
<p>- additional logic: SOC loop to avoid SOC drift</p>
<p>- further ON/OFF control to switch OFF the engine when the average power is too low to permit efficient operation.</p>
</html>"),
        __OpenModelica_commandLineOptions = "");
    end SHEVpowerFiltSocOO;
  end sHEV;

  package PSD
    model PSecu1 "Full Power Split Device power train using Map-Based components"
      import Modelica.Constants.*;
      extends Modelica.Icons.Example;
      parameter Modelica.SIunits.AngularVelocity wIceStart = 50;
      Modelica.SIunits.Energy EbatDel "energy delivered by the battery";
      Modelica.SIunits.Energy EgenDelM "energy delivered by gen trough mechanical flange";
      Modelica.SIunits.Energy Eroad "mechanical energy absorbed by roas (friction & air)";
      Modelica.SIunits.Energy EiceDel "mechanical energy delivered by ice";
      Modelica.SIunits.Energy Emot, Emass;
      Modelica.Mechanics.Rotational.Components.IdealPlanetary PSD(ratio = 78 / 30) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-50, 52})));
      Modelica.Mechanics.Rotational.Components.IdealGear idealGear(ratio = 3.905) annotation(
        Placement(transformation(extent = {{2, 42}, {22, 62}})));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor carVel annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {78, -12})));
      Modelica.Mechanics.Translational.Components.Mass mass(v(fixed = true, start = 0), m = 1300) annotation(
        Placement(transformation(extent = {{54, 42}, {74, 62}})));
      SupportModels.DragForce dragForce(fc = 0.014, rho = 1.226, S = 2.2, Cx = 0.26, m = mass.m) annotation(
        Placement(transformation(extent = {{-9, -9}, {9, 9}}, rotation = 90, origin = {89, 29})));
      wbEHPTlib.MapBased.IceConnP ice(wIceStart = wIceStart, mapsFileName = "PSDmaps.txt") annotation(
        Placement(transformation(extent = {{-98, 46}, {-78, 66}})));
      SupportModels.Batt1Conn bat(ECellMin = 0.9, ECellMax = 1.45, R0Cell = 0.0003, ns = 168, QCellNom = 2 * 6.5 * 3600.0, SOCInit = 0.6, ICellMax = 1e5, iCellEfficiency = 15 * 6.5) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-16, 0})));
      wbEHPTlib.SupportModels.ConnectorRelated.Conn d annotation(
        Placement(visible = true, transformation(extent = {{2, -40}, {28, -16}}, rotation = 0), iconTransformation(extent = {{4, -52}, {30, -28}}, rotation = 0)));
      wbEHPTlib.MapBased.ECUs.Ecu1 ECU(genLoopGain = 1.0) annotation(
        Placement(visible = true, transformation(origin = {-10, -41}, extent = {{-10, -9}, {10, 9}}, rotation = 0)));
      wbEHPTlib.MapBased.PsdMotConn mot(mapsFileName = "PSDmaps.txt", effMapName = "motEffTable") annotation(
        Placement(visible = true, transformation(extent = {{-28, 62}, {-8, 42}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.IdealRollingWheel wheel(radius = 0.31) annotation(
        Placement(visible = true, transformation(origin = {38, 52}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Electrical.Analog.Basic.Ground ground annotation(
        Placement(visible = true, transformation(origin = {10, 26}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
      wbEHPTlib.MapBased.PsdGenConn gen(mapsFileName = "PSDmaps.txt", effMapName = "genEffTable") annotation(
        Placement(visible = true, transformation(extent = {{-38, 10}, {-58, 30}}, rotation = 0)));
      SupportModels.PropDriver driver(yMax = 1.8, CycleFileName = "nedc.txt", k = 1) annotation(
        Placement(visible = true, transformation(extent = {{-60, -50}, {-40, -30}}, rotation = 0)));
    equation
      connect(ECU.tauRef, driver.tauRef) annotation(
        Line(points = {{-22, -41}, {-29, -41}, {-29, -40}, {-39, -40}}, color = {0, 0, 127}));
      connect(carVel.v, driver.V) annotation(
        Line(points = {{78, -23}, {78, -58}, {-50, -58}, {-50, -51.2}}, color = {0, 0, 127}));
      connect(mot.conn, ECU.conn1) annotation(
        Line(points = {{-27.2, 59.8}, {-27.2, 72}, {50, 72}, {50, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(gen.pin_p, bat.p) annotation(
        Line(points = {{-52, 30}, {-52, 30}, {-52, 34}, {-24, 34}, {-24, 10}, {-23.75, 10}}, color = {0, 0, 255}));
      connect(gen.pin_n, bat.n) annotation(
        Line(points = {{-42.2, 30}, {-8.5, 30}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(gen.flange_a, PSD.sun) annotation(
        Line(points = {{-58, 20}, {-58, 20}, {-70, 20}, {-70, 52}, {-60, 52}}));
      connect(gen.conn, ECU.conn1) annotation(
        Line(points = {{-58, 12.2}, {-58, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ground.p, bat.n) annotation(
        Line(points = {{0, 26}, {-8.5, 26}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(wheel.flangeT, mass.flange_a) annotation(
        Line(points = {{48, 52}, {54, 52}}, color = {0, 127, 0}));
      connect(wheel.flangeR, idealGear.flange_b) annotation(
        Line(points = {{28, 52}, {22, 52}}));
      connect(PSD.ring, mot.flange_a) annotation(
        Line(points = {{-40, 52}, {-34, 52}, {-28, 52}}));
      connect(idealGear.flange_a, mot.flange_b) annotation(
        Line(points = {{2, 52}, {-4, 52}, {-4, 52.2}, {-8, 52.2}}));
      connect(mot.pin_p, bat.p) annotation(
        Line(points = {{-22, 42.2}, {-22, 10}, {-23.75, 10}}, color = {0, 0, 255}));
      connect(mot.pin_n, bat.n) annotation(
        Line(points = {{-14, 42}, {-14, 10.1}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(bat.conn, ECU.conn1) annotation(
        Line(points = {{-15.75, -10}, {-16, -10}, {-16, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ice.conn, ECU.conn1) annotation(
        Line(points = {{-88, 46.2}, {-88, 46.2}, {-88, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ECU.conn1, d) annotation(
        Line(points = {{-10, -32.18}, {-10, -28}, {15, -28}}, color = {255, 204, 51}, thickness = 0.5));
      der(EbatDel) = (bat.p.v - bat.n.v) * bat.n.i;
      der(EgenDelM) = gen.pin_p.i * (gen.pin_p.v - gen.pin_n.v) + gen.flange_a.tau * der(gen.flange_a.phi);
      der(Eroad) = dragForce.flange.f * der(dragForce.flange.s);
      der(EiceDel) = -ice.flange_a.tau * der(ice.flange_a.phi);
      der(Emot) = mot.flange_a.tau * der(mot.flange_a.phi) + mot.flange_b.tau * der(mot.flange_b.phi);
      Emass = 0.5 * mass.m * der(mass.flange_a.s) ^ 2;
      connect(PSD.carrier, ice.flange_a) annotation(
        Line(points = {{-60, 56}, {-78, 56}}, color = {0, 0, 0}, smooth = Smooth.None));
      connect(dragForce.flange, mass.flange_b) annotation(
        Line(points = {{89, 38}, {90, 38}, {90, 52}, {74, 52}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(carVel.flange, mass.flange_b) annotation(
        Line(points = {{78, -2}, {78, 52}, {74, 52}}, color = {0, 127, 0}, smooth = Smooth.None));
      annotation(
        __Dymola_experimentSetupOutput,
        Documentation(info = "<html>
<p>This model simulates a PSD based power train with a simple control logic, in its ECU:</p>
<p>It tries to make the ICE deliver the average load power, at its optimal speed.</p>
<p>This has two main inconveniences:</p>
<ul>
<li>the battery SOC is not controlled and tends to drift</li>
<li>in urban environments the power is too low to allow efficient drive without shutting off the engine.</li>
</ul>
<p>These inconveniences are addressed in subsequent models PSEcu2 and PSecu3 (see their infos).</p>
</html>"),
        Diagram(coordinateSystem(extent = {{-100, -60}, {100, 80}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        experiment(StartTime = 0, StopTime = 1400, Tolerance = 0.0001, Interval = 2.8));
    end PSecu1;

    model PSecu2 "Full Power Split Device power train using Map-Based components"
      import Modelica.Constants.*;
      extends Modelica.Icons.Example;
      parameter Modelica.SIunits.AngularVelocity wIceStart = 50;
      Modelica.SIunits.Energy EbatDel "energy delivered by the battery";
      Modelica.SIunits.Energy EgenDelM "energy delivered by gen trough mechanical flange";
      Modelica.SIunits.Energy Eroad "mechanical energy absorbed by roas (friction & air)";
      Modelica.SIunits.Energy EiceDel "mechanical energy delivered by ice";
      Modelica.SIunits.Energy Emot, Emass;
      Modelica.Mechanics.Rotational.Components.IdealPlanetary PSD(ratio = 78 / 30) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-50, 52})));
      Modelica.Mechanics.Rotational.Components.IdealGear idealGear(ratio = 3.905) annotation(
        Placement(transformation(extent = {{2, 42}, {22, 62}})));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor carVel annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {78, -12})));
      Modelica.Mechanics.Translational.Components.Mass mass(v(fixed = true, start = 0), m = 1300) annotation(
        Placement(visible = true, transformation(extent = {{54, 42}, {74, 62}}, rotation = 0)));
      SupportModels.DragForce dragForce(fc = 0.014, rho = 1.226, S = 2.2, Cx = 0.26, m = mass.m) annotation(
        Placement(transformation(extent = {{-9, -9}, {9, 9}}, rotation = 90, origin = {89, 29})));
      wbEHPTlib.MapBased.IceConnP ice(wIceStart = wIceStart, mapsFileName = "PSDmaps.txt") annotation(
        Placement(transformation(extent = {{-98, 46}, {-78, 66}})));
      SupportModels.Batt1Conn bat(ECellMin = 0.9, ECellMax = 1.45, R0Cell = 0.0003, ns = 168, QCellNom = 2 * 6.5 * 3600.0, SOCInit = 0.6, ICellMax = 1e5, iCellEfficiency = 15 * 6.5) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-16, 0})));
      wbEHPTlib.SupportModels.ConnectorRelated.Conn d annotation(
        Placement(visible = true, transformation(extent = {{2, -40}, {28, -16}}, rotation = 0), iconTransformation(extent = {{4, -52}, {30, -28}}, rotation = 0)));
      wbEHPTlib.MapBased.ECUs.Ecu2 ECU(socLoopGain = 1e5, genLoopGain = 1.0) annotation(
        Placement(visible = true, transformation(origin = {-10, -41}, extent = {{-10, -9}, {10, 9}}, rotation = 0)));
      wbEHPTlib.MapBased.PsdMotConn mot(mapsFileName = "PSDmaps.txt", effMapName = "motEffTable") annotation(
        Placement(visible = true, transformation(extent = {{-28, 62}, {-8, 42}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.IdealRollingWheel wheel(radius = 0.31) annotation(
        Placement(visible = true, transformation(origin = {38, 52}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Electrical.Analog.Basic.Ground ground annotation(
        Placement(visible = true, transformation(origin = {10, 26}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
      wbEHPTlib.MapBased.PsdGenConn gen(mapsFileName = "PSDmaps.txt", effMapName = "genEffTable") annotation(
        Placement(visible = true, transformation(extent = {{-38, 10}, {-58, 30}}, rotation = 0)));
      SupportModels.PropDriver driver(yMax = 1.8, CycleFileName = "nedc.txt", k = 1) annotation(
        Placement(visible = true, transformation(extent = {{-60, -50}, {-40, -30}}, rotation = 0)));
    equation
      connect(carVel.flange, mass.flange_b) annotation(
        Line(points = {{78, -2}, {78, 52}, {74, 52}}, color = {0, 127, 0}));
      connect(dragForce.flange, mass.flange_b) annotation(
        Line(points = {{89, 38}, {90, 38}, {90, 52}, {74, 52}}, color = {0, 127, 0}));
      connect(wheel.flangeT, mass.flange_a) annotation(
        Line(points = {{48, 52}, {54, 52}}, color = {0, 127, 0}));
      connect(ECU.tauReference, driver.tauRef) annotation(
        Line(points = {{-22, -41}, {-29, -41}, {-29, -40}, {-39, -40}}, color = {0, 0, 127}));
      connect(carVel.v, driver.V) annotation(
        Line(points = {{78, -23}, {78, -58}, {-50, -58}, {-50, -51.2}}, color = {0, 0, 127}));
      connect(mot.conn, ECU.conn1) annotation(
        Line(points = {{-27.2, 59.8}, {-27.2, 72}, {50, 72}, {50, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(gen.pin_n, bat.n) annotation(
        Line(points = {{-42.2, 30}, {-8.5, 30}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(gen.flange_a, PSD.sun) annotation(
        Line(points = {{-58, 20}, {-58, 20}, {-70, 20}, {-70, 52}, {-60, 52}}));
      connect(gen.conn, ECU.conn1) annotation(
        Line(points = {{-58, 12.2}, {-58, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ground.p, bat.n) annotation(
        Line(points = {{0, 26}, {-8.5, 26}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(wheel.flangeR, idealGear.flange_b) annotation(
        Line(points = {{28, 52}, {22, 52}}));
      connect(PSD.ring, mot.flange_a) annotation(
        Line(points = {{-40, 52}, {-34, 52}, {-28, 52}}));
      connect(idealGear.flange_a, mot.flange_b) annotation(
        Line(points = {{2, 52}, {-4, 52}, {-4, 52.2}, {-8, 52.2}}));
      connect(mot.pin_p, bat.p) annotation(
        Line(points = {{-22, 42.2}, {-22, 10}, {-23.75, 10}}, color = {0, 0, 255}));
      connect(mot.pin_n, bat.n) annotation(
        Line(points = {{-14, 42}, {-14, 10.1}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(bat.conn, ECU.conn1) annotation(
        Line(points = {{-15.75, -10}, {-16, -10}, {-16, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ice.conn, ECU.conn1) annotation(
        Line(points = {{-88, 46.2}, {-88, 46.2}, {-88, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ECU.conn1, d) annotation(
        Line(points = {{-10, -32.18}, {-10, -28}, {15, -28}}, color = {255, 204, 51}, thickness = 0.5));
      der(EbatDel) = (bat.p.v - bat.n.v) * bat.n.i;
      der(EgenDelM) = gen.pin_p.i * (gen.pin_p.v - gen.pin_n.v) + gen.flange_a.tau * der(gen.flange_a.phi);
      der(Eroad) = dragForce.flange.f * der(dragForce.flange.s);
      der(EiceDel) = -ice.flange_a.tau * der(ice.flange_a.phi);
      der(Emot) = mot.flange_a.tau * der(mot.flange_a.phi) + mot.flange_b.tau * der(mot.flange_b.phi);
      Emass = 0.5 * mass.m * der(mass.flange_a.s) ^ 2;
      connect(PSD.carrier, ice.flange_a) annotation(
        Line(points = {{-60, 56}, {-78, 56}}, color = {0, 0, 0}, smooth = Smooth.None));
      connect(gen.pin_p, bat.p) annotation(
        Line(points = {{-52, 30}, {-52, 34}, {-22, 34}, {-22, 10}, {-23.75, 10}}, color = {0, 0, 255}));
      annotation(
        __Dymola_experimentSetupOutput,
        Documentation(info = "<html>
<p>This model simulates a PSD based power train with a simple control logic, in its ECU:</p>
<p>it tries to make the ICE deliver the average load power, at its optimal speed and with a loop in the ECU that compensates SOC drift (improvement of Ecu2 over Ecu1, thus of PSecu2 over PSecu1).</p>
</html>"),
        Diagram(coordinateSystem(extent = {{-100, -60}, {100, 80}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        experiment(StartTime = 0, StopTime = 1400, Tolerance = 0.0001, Interval = 2.8));
    end PSecu2;

    model PSecu3 "Full Power Split Device power train using Map-Based components"
      import Modelica.Constants.*;
      extends Modelica.Icons.Example;
      parameter Modelica.SIunits.AngularVelocity wIceStart = 50;
      parameter Real factorDebug = 100;
      // rad/s
      Modelica.SIunits.Energy EbatDel "energy delivered by the battery";
      Modelica.SIunits.Energy EgenDelM "energy delivered by gen trough mechanical flange";
      Modelica.SIunits.Energy Eroad "mechanical energy absorbed by roas (friction & air)";
      Modelica.SIunits.Energy EiceDel "mechanical energy delivered by ice";
      Modelica.SIunits.Energy Emot, Emass;
      Modelica.Mechanics.Rotational.Components.IdealPlanetary PSD(ratio = 78 / 30) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-50, 52})));
      Modelica.Mechanics.Rotational.Components.IdealGear idealGear(ratio = 3.905) annotation(
        Placement(transformation(extent = {{2, 42}, {22, 62}})));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor carVel annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {78, -12})));
      Modelica.Mechanics.Translational.Components.Mass mass(m = 1300, v(fixed = true, start = 0)) annotation(
        Placement(transformation(extent = {{54, 42}, {74, 62}})));
      SupportModels.DragForce dragForce(Cx = 0.26, S = 2.2, fc = 0.014, m = mass.m, rho = 1.226) annotation(
        Placement(transformation(extent = {{-9, -9}, {9, 9}}, rotation = 90, origin = {89, 29})));
      wbEHPTlib.MapBased.IceConnPOO ice(mapsFileName = "PSDmaps.txt", wIceStart = wIceStart) annotation(
        Placement(transformation(extent = {{-98, 46}, {-78, 66}})));
      SupportModels.Batt1Conn bat(ECellMin = 0.9, ECellMax = 1.45, R0Cell = 0.0003, ns = 168, QCellNom = 2 * 6.5 * 3600.0, SOCInit = 0.6, ICellMax = 1e5, iCellEfficiency = 15 * 6.5) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-16, 0})));
      wbEHPTlib.SupportModels.ConnectorRelated.Conn d annotation(
        Placement(visible = true, transformation(extent = {{2, -40}, {28, -16}}, rotation = 0), iconTransformation(extent = {{4, -52}, {30, -28}}, rotation = 0)));
      wbEHPTlib.MapBased.ECUs.Ecu3 ECU(genLoopGain = 1.0, socRef = 0.65) annotation(
        Placement(visible = true, transformation(origin = {-10, -41}, extent = {{-10, -9}, {10, 9}}, rotation = 0)));
      wbEHPTlib.MapBased.PsdMotConn mot(mapsFileName = "PSDmaps.txt", effMapName = "motEffTable") annotation(
        Placement(visible = true, transformation(extent = {{-28, 62}, {-8, 42}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.IdealRollingWheel wheel(radius = 0.31) annotation(
        Placement(visible = true, transformation(origin = {38, 52}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      wbEHPTlib.MapBased.PsdGenConn gen(mapsFileName = "PSDmaps.txt", effMapName = "genEffTable") annotation(
        Placement(visible = true, transformation(extent = {{-38, 10}, {-58, 30}}, rotation = 0)));
      SupportModels.PropDriver driver(yMax = 1.8, CycleFileName = "nedc.txt", k = 1) annotation(
        Placement(visible = true, transformation(extent = {{-60, -50}, {-40, -30}}, rotation = 0)));
  Modelica.Electrical.Analog.Basic.Ground ground annotation(
        Placement(visible = true, transformation(origin = {12, 10}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    equation
      connect(ground.p, bat.n) annotation(
        Line(points = {{12, 20}, {-8, 20}, {-8, 10}, {-8, 10}, {-8, 10}}, color = {0, 0, 255}));
      connect(ECU.tauReference, driver.tauRef) annotation(
        Line(points = {{-22, -41}, {-29, -41}, {-29, -40}, {-39, -40}}, color = {0, 0, 127}));
      connect(carVel.v, driver.V) annotation(
        Line(points = {{78, -23}, {78, -58}, {-50, -58}, {-50, -51.2}}, color = {0, 0, 127}));
      connect(mot.conn, ECU.conn) annotation(
        Line(points = {{-27.2, 59.8}, {-27.2, 72}, {50, 72}, {50, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(gen.pin_p, bat.p) annotation(
        Line(points = {{-52, 30}, {-52, 30}, {-52, 34}, {-22, 34}, {-22, 10}, {-23.75, 10}}, color = {0, 0, 255}));
      connect(gen.pin_n, bat.n) annotation(
        Line(points = {{-42.2, 30}, {-8.5, 30}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(gen.flange_a, PSD.sun) annotation(
        Line(points = {{-58, 20}, {-58, 20}, {-70, 20}, {-70, 52}, {-60, 52}}));
      connect(gen.conn, ECU.conn) annotation(
        Line(points = {{-58, 12.2}, {-58, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(wheel.flangeT, mass.flange_a) annotation(
        Line(points = {{48, 52}, {54, 52}}, color = {0, 127, 0}));
      connect(wheel.flangeR, idealGear.flange_b) annotation(
        Line(points = {{28, 52}, {22, 52}}));
      connect(PSD.ring, mot.flange_a) annotation(
        Line(points = {{-40, 52}, {-34, 52}, {-28, 52}}));
      connect(idealGear.flange_a, mot.flange_b) annotation(
        Line(points = {{2, 52}, {-4, 52}, {-4, 52.2}, {-8, 52.2}}));
      connect(mot.pin_p, bat.p) annotation(
        Line(points = {{-22, 42.2}, {-22, 10}, {-23.75, 10}}, color = {0, 0, 255}));
      connect(mot.pin_n, bat.n) annotation(
        Line(points = {{-14, 42}, {-14, 10.1}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(bat.conn, ECU.conn) annotation(
        Line(points = {{-15.75, -10}, {-16, -10}, {-16, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ice.conn, ECU.conn) annotation(
        Line(points = {{-88, 46.2}, {-88, 46.2}, {-88, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ECU.conn, d) annotation(
        Line(points = {{-10, -32.18}, {-10, -28}, {15, -28}}, color = {255, 204, 51}, thickness = 0.5));
      der(EbatDel) = (bat.p.v - bat.n.v) * bat.n.i;
      der(EgenDelM) = gen.pin_p.i * (gen.pin_p.v - gen.pin_n.v) + gen.flange_a.tau * der(gen.flange_a.phi);
      der(Eroad) = dragForce.flange.f * der(dragForce.flange.s);
      der(EiceDel) = -ice.flange_a.tau * der(ice.flange_a.phi);
      der(Emot) = mot.flange_a.tau * der(mot.flange_a.phi) + mot.flange_b.tau * der(mot.flange_b.phi);
      Emass = 0.5 * mass.m * der(mass.flange_a.s) ^ 2;
      connect(PSD.carrier, ice.flange_a) annotation(
        Line(points = {{-60, 56}, {-78, 56}}, color = {0, 0, 0}, smooth = Smooth.None));
      connect(dragForce.flange, mass.flange_b) annotation(
        Line(points = {{89, 38}, {90, 38}, {90, 52}, {74, 52}}, color = {0, 127, 0}, smooth = Smooth.None));
      connect(carVel.flange, mass.flange_b) annotation(
        Line(points = {{78, -2}, {78, 52}, {74, 52}}, color = {0, 127, 0}, smooth = Smooth.None));
      annotation(
        __Dymola_experimentSetupOutput,
        Documentation(info = "<html>
<p>This model simulates a PSD based power train with a simple control logic, in its ECU:</p>
<p>it tries to make the ICE deliver the average load power, at its optimal speed and:</p>
<ol>
<li>has a loop in the ECU that compensates SOC drift (improvement Ecu1, thus over PSecu1)</li>
<li>has an hysteresis control in the ECU that implements ON/OFF strategy to avoid too low powers (and correspondingly high consumption) in urban environments (improvement of Ecu3 over Ecu1 and Ecu2, thus of PSecu3 over PSecu1 and PSecu2)</li>
</ol>
</html>"),
        Diagram(coordinateSystem(extent = {{-100, -60}, {100, 80}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        experiment(StartTime = 0, StopTime = 1400, Tolerance = 0.0001, Interval = 2.8));
    end PSecu3;

    model PSecu2PA "Full Power Split Device power train using Map-Based components"
      import Modelica.Constants.*;
      extends Modelica.Icons.Example;
      parameter Modelica.SIunits.AngularVelocity wIceStart = 50;
      Modelica.SIunits.Energy EbatDel "energy delivered by the battery";
      Modelica.SIunits.Energy EgenDelM "energy delivered by gen trough mechanical flange";
      Modelica.SIunits.Energy Eroad "mechanical energy absorbed by roas (friction & air)";
      Modelica.SIunits.Energy EiceDel "mechanical energy delivered by ice";
      Modelica.SIunits.Energy Emot, Emass;
      Modelica.Mechanics.Rotational.Components.IdealPlanetary PSD(ratio = 78 / 30) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 0, origin = {-50, 52})));
      Modelica.Mechanics.Rotational.Components.IdealGear idealGear(ratio = 3.905) annotation(
        Placement(transformation(extent = {{2, 42}, {22, 62}})));
      Modelica.Mechanics.Translational.Sensors.SpeedSensor carVel annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 270, origin = {78, -12})));
      Modelica.Mechanics.Translational.Components.Mass mass(v(fixed = true, start = 0), m = 1300) annotation(
        Placement(visible = true, transformation(extent = {{54, 42}, {74, 62}}, rotation = 0)));
      SupportModels.DragForce dragForce(fc = 0.014, rho = 1.226, S = 2.2, Cx = 0.26, m = mass.m) annotation(
        Placement(transformation(extent = {{-9, -9}, {9, 9}}, rotation = 90, origin = {89, 29})));
      wbEHPTlib.MapBased.IceConnP ice(wIceStart = wIceStart, mapsFileName = "PSDmaps.txt") annotation(
        Placement(transformation(extent = {{-98, 46}, {-78, 66}})));
      SupportModels.Batt1Conn bat(ECellMin = 0.9, ECellMax = 1.45, R0Cell = 0.0003, SOCInit = 0.6, ICellMax = 1e5, iCellEfficiency = 15 * 6.5, ns = 100, QCellNom = 2 * 3600.0) annotation(
        Placement(transformation(extent = {{-10, -10}, {10, 10}}, rotation = 90, origin = {-16, 0})));
      wbEHPTlib.SupportModels.ConnectorRelated.Conn d annotation(
        Placement(visible = true, transformation(extent = {{2, -40}, {28, -16}}, rotation = 0), iconTransformation(extent = {{4, -52}, {30, -28}}, rotation = 0)));
      wbEHPTlib.MapBased.ECUs.Ecu2 ECU(genLoopGain = 1.0, socLoopGain = 2e4, powFiltT = 10) annotation(
        Placement(visible = true, transformation(origin = {-10, -41}, extent = {{-10, -9}, {10, 9}}, rotation = 0)));
      wbEHPTlib.MapBased.PsdMotConn mot(mapsFileName = "PSDmaps.txt", effMapName = "motEffTable") annotation(
        Placement(visible = true, transformation(extent = {{-28, 62}, {-8, 42}}, rotation = 0)));
      Modelica.Mechanics.Rotational.Components.IdealRollingWheel wheel(radius = 0.31) annotation(
        Placement(visible = true, transformation(origin = {38, 52}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
      Modelica.Electrical.Analog.Basic.Ground ground annotation(
        Placement(visible = true, transformation(origin = {10, 26}, extent = {{10, 10}, {-10, -10}}, rotation = 270)));
      wbEHPTlib.MapBased.PsdGenConn gen(mapsFileName = "PSDmaps.txt", effMapName = "genEffTable") annotation(
        Placement(visible = true, transformation(extent = {{-38, 10}, {-58, 30}}, rotation = 0)));
      SupportModels.PropDriver driver(yMax = 1.8, CycleFileName = "nedc.txt", k = 1) annotation(
        Placement(visible = true, transformation(extent = {{-60, -50}, {-40, -30}}, rotation = 0)));
    equation
      connect(carVel.flange, mass.flange_b) annotation(
        Line(points = {{78, -2}, {78, 52}, {74, 52}}, color = {0, 127, 0}));
      connect(dragForce.flange, mass.flange_b) annotation(
        Line(points = {{89, 38}, {90, 38}, {90, 52}, {74, 52}}, color = {0, 127, 0}));
      connect(wheel.flangeT, mass.flange_a) annotation(
        Line(points = {{48, 52}, {54, 52}}, color = {0, 127, 0}));
      connect(ECU.tauReference, driver.tauRef) annotation(
        Line(points = {{-22, -41}, {-29, -41}, {-29, -40}, {-39, -40}}, color = {0, 0, 127}));
      connect(carVel.v, driver.V) annotation(
        Line(points = {{78, -23}, {78, -58}, {-50, -58}, {-50, -51.2}}, color = {0, 0, 127}));
      connect(mot.conn, ECU.conn1) annotation(
        Line(points = {{-27.2, 59.8}, {-27.2, 72}, {50, 72}, {50, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(gen.pin_n, bat.n) annotation(
        Line(points = {{-42.2, 30}, {-8.5, 30}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(gen.flange_a, PSD.sun) annotation(
        Line(points = {{-58, 20}, {-58, 20}, {-70, 20}, {-70, 52}, {-60, 52}}));
      connect(gen.conn, ECU.conn1) annotation(
        Line(points = {{-58, 12.2}, {-58, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ground.p, bat.n) annotation(
        Line(points = {{0, 26}, {-8.5, 26}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(wheel.flangeR, idealGear.flange_b) annotation(
        Line(points = {{28, 52}, {22, 52}}));
      connect(PSD.ring, mot.flange_a) annotation(
        Line(points = {{-40, 52}, {-34, 52}, {-28, 52}}));
      connect(idealGear.flange_a, mot.flange_b) annotation(
        Line(points = {{2, 52}, {-4, 52}, {-4, 52.2}, {-8, 52.2}}));
      connect(mot.pin_p, bat.p) annotation(
        Line(points = {{-22, 42.2}, {-22, 10}, {-23.75, 10}}, color = {0, 0, 255}));
      connect(mot.pin_n, bat.n) annotation(
        Line(points = {{-14, 42}, {-14, 10.1}, {-8.5, 10.1}}, color = {0, 0, 255}));
      connect(bat.conn, ECU.conn1) annotation(
        Line(points = {{-15.75, -10}, {-16, -10}, {-16, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ice.conn, ECU.conn1) annotation(
        Line(points = {{-88, 46.2}, {-88, 46.2}, {-88, -20}, {-10, -20}, {-10, -32.18}}, color = {255, 204, 51}, thickness = 0.5));
      connect(ECU.conn1, d) annotation(
        Line(points = {{-10, -32.18}, {-10, -28}, {15, -28}}, color = {255, 204, 51}, thickness = 0.5));
      der(EbatDel) = (bat.p.v - bat.n.v) * bat.n.i;
      der(EgenDelM) = gen.pin_p.i * (gen.pin_p.v - gen.pin_n.v) + gen.flange_a.tau * der(gen.flange_a.phi);
      der(Eroad) = dragForce.flange.f * der(dragForce.flange.s);
      der(EiceDel) = -ice.flange_a.tau * der(ice.flange_a.phi);
      der(Emot) = mot.flange_a.tau * der(mot.flange_a.phi) + mot.flange_b.tau * der(mot.flange_b.phi);
      Emass = 0.5 * mass.m * der(mass.flange_a.s) ^ 2;
      connect(PSD.carrier, ice.flange_a) annotation(
        Line(points = {{-60, 56}, {-78, 56}}, color = {0, 0, 0}, smooth = Smooth.None));
      connect(gen.pin_p, bat.p) annotation(
        Line(points = {{-52, 30}, {-52, 34}, {-22, 34}, {-22, 10}, {-23.75, 10}}, color = {0, 0, 255}));
      annotation(
        __Dymola_experimentSetupOutput,
        Documentation(info = "<html>
<p>Like PSecu2, but solves the Proposed Activity that requires a very small battery and near-CVT (continuosuly-variable Transmission) operation. </p>
</html>"),
        Diagram(coordinateSystem(extent = {{-100, -60}, {100, 80}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = false, initialScale = 0.1, grid = {2, 2})),
        experiment(StartTime = 0, StopTime = 1400, Tolerance = 0.0001, Interval = 2.8));
    end PSecu2PA;
  end PSD;
  annotation(
    uses(Modelica(version = "3.2.2")));
end wbEVPkg;

